package es.verdnatura.presentation.view.feature.ajustes.fragment

import android.app.AlertDialog
import android.content.ClipData
import android.content.ClipboardManager
import android.content.Context
import android.content.Context.CLIPBOARD_SERVICE
import android.content.SharedPreferences
import android.os.Bundle
import android.provider.Settings
import android.util.Log
import android.view.KeyEvent
import android.view.View
import androidx.appcompat.widget.TooltipCompat.setTooltipText
import androidx.preference.PreferenceManager
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentAjustesBinding
import es.verdnatura.domain.notNull
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnAjustesItemClickListener
import es.verdnatura.presentation.common.hideKeyboard
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.feature.ajustes.adapter.AjustesAdapter
import es.verdnatura.presentation.view.feature.ajustes.model.AjustesItemVO
import es.verdnatura.presentation.view.feature.ajustes.model.SectorItemVO
import es.verdnatura.presentation.view.feature.main.activity.MainActivity
import es.verdnatura.presentation.view.feature.pasillero.model.PasillerosItemVO
import timber.log.Timber.d

class AjustesFragment :
    BaseFragment<FragmentAjustesBinding, AjustesViewModel>(AjustesViewModel::class) {

    private lateinit var customDialog: CustomDialog
    private var sectorListVO: List<SectorItemVO> = listOf()
    private var vozList: List<String> = listOf()
    private var prefs: SharedPreferences? = null
    private var ajustesAdapter: AjustesAdapter? = null

    //añadido
    private var wagonList: List<String> = listOf()
    private var tagsList: List<String> = listOf()

    companion object {
        fun newInstance() = AjustesFragment()
    }

    override fun getLayoutId(): Int = R.layout.fragment_ajustes


    override fun onCreate(savedInstanceState: Bundle?) {
        prefs = requireActivity().getSharedPreferences(PREFS_USER, 0)
        customDialog = CustomDialog(requireContext())
        viewModel.inititializeDefaultAjusts(
            getData(SECTORDESCRIP),
            getData(SECTORFK).toInt(),
            getData(WAREHOUSEFK).toInt(),
            getData(VOZ),
            getData(WAGON),
            getData(TAGSTYPE)


        )
        super.onCreate(savedInstanceState)

    }


    override fun init() {

        ma.hideBottomNavigation(View.VISIBLE)
        binding.userText.setText(getData(USER))
        val versionName = requireActivity().packageManager.getPackageInfo(
            requireActivity().packageName,
            0
        ).versionName!!
        binding.itemVersion.setText(versionName)

        binding.androididText.setText(
            getData(ANDROID_ID)
        )
        binding.txtserver.setText(getData("base_url"))
        Log.d("VERDNATURA::","La base es "+getData("base_url"))

        super.init()

        setEvents()

    }

    fun setEvents() {
        binding.androididText.setOnClickListener { copyanddrag(binding.androididText.text.toString()) }
        binding.androididText.setOnLongClickListener { copyanddrag(binding.androididText.text.toString()) }


    }


    override fun observeViewModel() {
        with(viewModel) {
            ajustesAdapter = AjustesAdapter(ajustesitem, object : OnAjustesItemClickListener {
                override fun onAjustesItemClickListener(item: AjustesItemVO) {
                    if (item.id == 0) {
                        binding.splashProgress.visibility = View.VISIBLE
                       // viewModel.getSectors(getData(USER),getData(PASSWORD))
                        viewModel.getSectorsSalix(getData(TOKEN))
                    } else if (item.id == 5) {
                        requireActivity().onBackPressed()
                    } else if (item.id == 1) {
                        val listCarros: ArrayList<String> = ArrayList()
                        listCarros.add("1")
                        listCarros.add("2")
                        listCarros.add("3")
                        listCarros.add("4")
                        listCarros.add("5")
                        listCarros.add("6")
                        wagonList = listCarros

                        val array = arrayOfNulls<String>(listCarros.size)

                        showDialogCarros(array = listCarros.toArray(array))


                    } else if (item.id == 2) {
                        val listVoz: ArrayList<String> = ArrayList()
                        listVoz.add("NO")
                        listVoz.add("YES")
                        vozList = listVoz
                        val array = arrayOfNulls<String>(listVoz.size)
                        showDialogVoz(array = listVoz.toArray(array))

                    } else if (item.id == 3) {
                        val listTagsType: ArrayList<String> = ArrayList()
                        listTagsType.add("Stickers")
                        listTagsType.add("SmartTags")
                        tagsList = listTagsType
                        val array = arrayOfNulls<String>(listTagsType.size)
                        showItemPickerType(array = listTagsType.toArray(array))
                    } else if (item.id == 4) {
                        ma.onPasillerosItemClickListener(
                            PasillerosItemVO(
                                title = "Control de usuario de vehículo"
                            ), getData(USERFK)
                        )

                    }
                }
            })

            workerupdateSector.observe(viewLifecycleOwner,  {
                binding.splashProgress.visibility = View.GONE

                ma.messageWithSound(it.errorMessage,it.isError,false)
            })

            binding.ajustessItems.adapter = ajustesAdapter
            binding.ajustessItems.layoutManager =
                LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)

            loadSectorList.observe(viewLifecycleOwner, { event ->
                event.getContentIfNotHandled().notNull {
                    binding.splashProgress.visibility = View.INVISIBLE
                    if (it.list.isNotEmpty() && it.list.get(0).isError) {
                        customDialog.setTitle("Error").setDescription(it.list.get(0).errorMessage)
                            .setOkButton(getString(R.string.Close)) {
                                customDialog.dismiss()
                            }.show()
                    } else {

                        if (it.list != null && it.list.size > 0) {
                            val listSectores: ArrayList<String> = ArrayList()
                            it.list.forEach {
                                listSectores.add(it.description)
                            }
                            val array = arrayOfNulls<String>(listSectores.size)
                            sectorListVO = it.list
                            showDialogSector(listSectores.toArray(array))

                        } else {
                            customDialog.setTitle("Sectores")
                                .setDescription(getString(R.string.Noexistessectores))
                                .setOkButton(getString(R.string.Close)) {
                                    customDialog.dismiss()
                                }.show()
                        }

                    }
                }
            })
        }
        super.observeViewModel()
    }

    private fun showDialogSector(array: Array<String>) {
        val builder = AlertDialog.Builder(this.context)


        builder.setTitle(getString(R.string.Seleccionaunsector))
        builder.setItems(array) { _, which ->
            val selected = array[which]
            sectorListVO.forEach {
                if (it.description.equals(selected)) {
                    val editor = prefs!!.edit()
                    editor.putString(SECTORDESCRIP, it.description)
                    editor.putInt(SECTORFK, it.id)
                    it.warehouseFk?.let { it1 -> editor.putInt(WAREHOUSEFK, it1) }
                    editor.apply()

                    viewModel.ajustesitem.get(0).sectorFk = it.id
                    viewModel.ajustesitem.get(0).warehouse = it.warehouseFk
                    viewModel.ajustesitem.get(0).selected = it.description
                    d("El sector marcado es "+it.id.toString())
                    viewModel.worker_updateSector(getData(USER),getData(PASSWORD),it.id.toString())
                    ajustesAdapter!!.notifyDataSetChanged()

                    return@forEach
                }
            }

        }
        val dialog = builder.create()
        dialog.show()
    }

    private fun showDialogVoz(array: Array<String>) {
        val builder = AlertDialog.Builder(this.context)


        builder.setTitle(getString(R.string.Seleccionaunsector))
        builder.setItems(array) { _, which ->
            val selected = array[which]
            vozList.forEach {
                if (it.equals(selected)) {
                    val editor = prefs!!.edit()
                    editor.putString(VOZ, it)
                    editor.apply()
                    viewModel.ajustesitem.get(2).selected = it

                    ajustesAdapter!!.notifyDataSetChanged()

                    return@forEach
                }
            }


        }

        val dialog = builder.create()
        dialog.show()
    }

    private fun showDialogCarros(array: Array<String>) {
        val builder = AlertDialog.Builder(this.context)
        builder.setTitle(getString(R.string.Seleccionauncarro))
        builder.setItems(array) { _, which ->
            val selected = array[which]
            wagonList.forEach {
                if (it.equals(selected)) {
                    val editor = prefs!!.edit()
                    editor.putInt(WAGON, it.toInt())
                    editor.apply()

                    viewModel.ajustesitem.get(1).selected = it
                    ajustesAdapter!!.notifyDataSetChanged()

                    return@forEach
                }
            }


        }

        val dialog = builder.create()
        dialog.show()
    }

    private fun showItemPickerType(array: Array<String>) {
        val builder = AlertDialog.Builder(this.context)


        builder.setTitle(getString(R.string.typeTags))
        builder.setItems(array) { _, which ->
            val selected = array[which]
            tagsList.forEach {
                if (it.equals(selected)) {
                    val editor = prefs!!.edit()
                    editor.putString(TAGSTYPE, it)
                    editor.apply()

                    viewModel.ajustesitem.get(3).selected = it
                    ajustesAdapter!!.notifyDataSetChanged()

                    return@forEach
                }
            }


        }

        val dialog = builder.create()
        dialog.show()
    }


  /*  fun setDefaults(key: String?, value: String?, context: Context?) {
        val preferences: SharedPreferences = PreferenceManager.getDefaultSharedPreferences(context)
        val editor = preferences.edit()
        editor.putString(key, value)
        editor.commit()
    }*/

   /* private fun getDefaults(key: String?, context: Context?): String? {
        val preferences = PreferenceManager.getDefaultSharedPreferences(context)
        return preferences.getString(key, null)
    }*/


    private fun copyanddrag(string: String): Boolean {
        val clipboard = getActivity()?.getSystemService(CLIPBOARD_SERVICE) as ClipboardManager
        val clip: ClipData = ClipData.newPlainText("Android_id", string)
        clipboard.setPrimaryClip(clip)
        setTooltipText(binding.androididText, getString(R.string.copied))
        return false

    }

}
