package es.verdnatura.presentation.view.feature.ajustes.fragment


import android.content.Context
import android.util.Log
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.Transformations
import es.verdnatura.domain.GetAjustesUserCase
import es.verdnatura.presentation.base.BaseViewModel
import es.verdnatura.presentation.base.getMessageFromAllResponse
import es.verdnatura.presentation.base.nameofFunction
import es.verdnatura.presentation.common.Event
import es.verdnatura.presentation.common.ResponseItemVO
import es.verdnatura.presentation.view.feature.ajustes.model.AjustesItemVO
import es.verdnatura.presentation.view.feature.ajustes.model.SectorItemVO
import es.verdnatura.presentation.view.feature.ajustes.model.SectorListVO
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response
import timber.log.Timber.d


class AjustesViewModel(context: Context) : BaseViewModel() {

     private val getAjustesUserCase:GetAjustesUserCase = GetAjustesUserCase(context)
     val version : String = "5.0.0"

     private val _ajustesitem by lazy { ArrayList<AjustesItemVO>() }
     val ajustesitem: List<AjustesItemVO>
          get() = _ajustesitem

     private val _sectorList by lazy { MutableLiveData<SectorListVO>() }
     val sectorList: LiveData<SectorListVO>
          get() = _sectorList

     private val _workerupdateSector by lazy { MutableLiveData<ResponseItemVO>() }
     val workerupdateSector: LiveData<ResponseItemVO>
          get() = _workerupdateSector



     val loadSectorList = Transformations.map(_sectorList) {Event(it)}




     fun inititializeDefaultAjusts(sectorDescrip: String,sectorFk : Int, warehouseFk : Int,vozDescrip:String,carroDescrip:String,TypeTags:String) {
          _ajustesitem.add(
               AjustesItemVO(0,
                    "Sector",
                    sectorDescrip,
                    sectorFk,
                    warehouseFk)
          )


          _ajustesitem.add(
               AjustesItemVO(1,
                    "Carros",
                    carroDescrip,
                    0,
                    0)
          )
          /*_ajustesitem.add(
               AjustesItemVO(2,
                    "Voz",
                    vozDescrip,
                    0,
                    0)
          )*/
          _ajustesitem.add(
               AjustesItemVO(3,
                    "Tipo etiqueta",
                    TypeTags,
                    0,
                    0)
          )
          _ajustesitem.add(
               AjustesItemVO(4,
                    "Control de vehículos",
                    "",
                    0,
                    0)
          )
          _ajustesitem.add(
               AjustesItemVO(5,
                    "Cerrar Sesión",
                    "",
                    0,
                    0)
          )

     }

     fun getSectors(user:String,password:String){
          getAjustesUserCase.getSectors(user,password).enqueue(object : Callback<List<SectorItemVO>>{
               override fun onFailure(call: Call<List<SectorItemVO>>, t: Throwable) {
                    val listError:ArrayList<SectorItemVO> = ArrayList()
                    listError.add(SectorItemVO(0,"",0,true,t.message!!))
                    _sectorList.value = SectorListVO(listError)
               }

               override fun onResponse(
                    call: Call<List<SectorItemVO>>,
                    response: Response<List<SectorItemVO>>
               ) {
                    if (response.body() != null){
                         _sectorList.value = response.body()?.let { SectorListVO(it) }
                    }else{
                         val listError:ArrayList<SectorItemVO> = ArrayList()
                         listError.add(SectorItemVO(0,"",0,true,"Error en la llamada sector_get"))
                         _sectorList.value = SectorListVO(listError)
                    }
               }

          })
     }
     fun getSectorsSalix(token:String){
          getAjustesUserCase.getSectorsSalix(token).enqueue(object : Callback<List<SectorItemVO>>{
               override fun onFailure(call: Call<List<SectorItemVO>>, t: Throwable) {
                    val listError:ArrayList<SectorItemVO> = ArrayList()
                    listError.add(SectorItemVO(0,"",0,true,t.message!!))
                    _sectorList.value = SectorListVO(listError)
               }

               override fun onResponse(
                    call: Call<List<SectorItemVO>>,
                    response: Response<List<SectorItemVO>>
               ) {


                    if (response.body() != null){
                         _sectorList.value = response.body()?.let { SectorListVO(it) }
                    }else{
                         val listError:ArrayList<SectorItemVO> = ArrayList()
                         listError.add(SectorItemVO(0,"",0,true,
                              getMessageFromAllResponse(nameofFunction(this),response.message())
                         ))
                         _sectorList.value = SectorListVO(listError)
                    }
               }

          })
     }
     fun worker_updateSector(user:String,password:String,sectorFk:String){
          d("El sector marcado dentro es "+sectorFk)
          getAjustesUserCase.worker_updateSector(user,password,sectorFk).enqueue(object : Callback<Unit>{
               override fun onFailure(call: Call<Unit>, t: Throwable) {

                    _workerupdateSector.value = ResponseItemVO(isError = true, errorMessage = getMessageFromAllResponse(nameofFunction(this),t.message!!))
               }

               override fun onResponse(
                    call: Call<Unit>,
                    response: Response<Unit>
               ) {

                    if (!response.isSuccessful()){
                         _workerupdateSector.value=ResponseItemVO(
                              isError = true,
                              errorMessage = getMessageFromAllResponse(nameofFunction(this),response.message())
                         )
                    }else{
                         _workerupdateSector.value=ResponseItemVO(response="Actualizado sector",isError = false, errorMessage ="Actualizado sector")
                    }

                    /*if (response.body() != null){
                         _sectorList.value = response.body()?.let { SectorListVO(it) }
                    }else{
                         val listError:ArrayList<SectorItemVO> = ArrayList()
                         listError.add(SectorItemVO(0,"",0,true,"Error en la llamada sector_get"))
                         _sectorList.value = SectorListVO(listError)
                    }*/
               }

          })
     }

}
