package es.verdnatura.presentation.view.feature.collection.adapter

import android.content.Context
import android.graphics.Color
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.core.content.ContextCompat.getColor
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import es.verdnatura.R
import es.verdnatura.databinding.ItemArticleRowBinding
import es.verdnatura.domain.toast
import es.verdnatura.presentation.common.*
import es.verdnatura.presentation.view.feature.pasillero.model.PasillerosItemVO
import es.verdnatura.presentation.view.feature.sacador.model.SaleVO


class SaleAdapter(
    private val items: List<SaleVO>,
    private val onPasillerosItemClickListener: OnPasillerosItemClickListener,
    private val onQuantityClick: OnQuantityClickListener,
    private val onSaleClickListener: OnSaleClickListener,
    private val onMistakeClickListener: OnMistakeClickListener,
    private val onPackingClick: onPackingClickListener,
) : RecyclerView.Adapter<SaleAdapter.AjustesItemHolder>() {
    var context: Context? = null
    var position: Int = 0
    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): AjustesItemHolder {
        this.context = parent.context
        return AjustesItemHolder(
            ItemArticleRowBinding.inflate(LayoutInflater.from(parent.context), parent, false)
        )
    }

    override fun getItemCount() = items.size

    override fun onBindViewHolder(holder: AjustesItemHolder, position: Int) {
        this.position = position
        holder.bind(items[position])
    }

    inner class AjustesItemHolder(
        val binding: ItemArticleRowBinding
    ) : RecyclerView.ViewHolder(binding.root) {
        fun bind(sale: SaleVO) {
            binding.apply {
                if (sale.pickedQuantity.isNullOrEmpty())
                    sale.pickedQuantity = "0"

                val childLayoutManager =
                    LinearLayoutManager(context!!, RecyclerView.HORIZONTAL, false)

                itemArticlePlacements.apply {
                    layoutManager = childLayoutManager
                    adapter = PlacementAdapter(sale.placements, onPasillerosItemClickListener)
                }

                //CLICK EVENTS
                contentLayout.setOnClickListener {
                    onSaleClickListener.onSaleClick(sale)
                }

                itemArticleItemFk.setOnClickListener {
                    onPasillerosItemClickListener.onPasillerosItemClickListener(
                        PasillerosItemVO(
                            title = "Consultar artículo"
                        ), sale.itemFk
                    )
                }

                itemPackingItemFk.setOnClickListener {
                    onPackingClick.onPackingClick(sale)
                }

                itemArticleQuantity.setOnClickListener {
                    onQuantityClick.onQuantityClick(sale)
                }

                itemArticleQuantityPicked.setOnClickListener {
                    onQuantityClick.onQuantityClick(sale)
                }

                txtde.setOnClickListener {
                    onQuantityClick.onQuantityClick(sale)
                }

                contentLayout.setOnLongClickListener {
                    "Nueva funcionalidad en el icono de la derecha".toast(context!!)
                    //onMistakeClickListener.onMistakeClickListener(sale)
                    false
                }

                /*itemRowLayout.setOnLongClickListener{
                    onMistakeClickListener.onMistakeClickListener(sale)
                    false
                }*/
                imageErrorMessage.setOnClickListener {
                    onMistakeClickListener.onMistakeClickListener(sale)
                    false
                }


                //ERROR
                if (sale.originalQuantity != sale.quantity) {
                    layoutError.visibility = View.VISIBLE
                    txtError.text = "Cantidad original: " + sale.originalQuantity
                    if (sale.isPrepared == "1" || sale.isControlled == "1")
                        sale.pickedQuantity = sale.quantity
                }

                if (sale.isNew) {
                    layoutError.visibility = View.VISIBLE
                    txtError.text = "Artículo nuevo"

                }

                paintTicketcolor(sale.level, itemTicketColor)


                if (!sale.isNew && sale.originalQuantity == sale.quantity) {
                    layoutError.visibility = View.GONE
                }


                //SEMAFORO
                if (sale.isPreviousPrepared == "1") {
                    itemArticleRowSemaforoPre.setBackgroundColor(
                        getColor(
                            context!!,
                            R.color.verdnatura_dark_sky_blue
                        )
                    )
                } else {
                    itemArticleRowSemaforoPre.setBackgroundColor(
                        getColor(
                            context!!,
                            R.color.verdnatura_warm_grey
                        )
                    )
                }

                if (sale.isPrepared == "1") {
                    itemArticleRowSemaforoSac.setBackgroundColor(
                        getColor(
                            context!!,
                            R.color.verdnatura_pumpkin_orange
                        )
                    )
                } else {
                    itemArticleRowSemaforoSac.setBackgroundColor(
                        getColor(
                            context!!,
                            R.color.verdnatura_warm_grey
                        )
                    )
                }



                if (sale.isControlled == "1") {
                    itemArticleRowSemaforoCon.setBackgroundColor(
                        getColor(
                            context!!,
                            R.color.verdnatura_dark_mint
                        )
                    )
                } else {
                    itemArticleRowSemaforoCon.setBackgroundColor(
                        getColor(
                            context!!,
                            R.color.verdnatura_warm_grey
                        )
                    )
                }


                if (sale.isControlled == "1") {
                    contentLayout.setBackgroundColor(
                        getColor(
                            context!!,
                            R.color.verdnatura_dark_mint
                        )
                    )
                } else if (sale.isPrepared == "1") {
                    contentLayout.setBackgroundColor(
                        getColor(
                            context!!,
                            R.color.verdnatura_pumpkin_orange
                        )
                    )
                } /*else if (sale.isPreviousPrepared == "1") {
                    contentLayout.setBackgroundColor(
                        getColor(
                            context!!,
                            R.color.verdnatura_dark_sky_blue
                        )
                    )
                }*/ else if (sale.isPreControlled == "1") {

                    contentLayout.setBackgroundColor(
                        getColor(
                            context!!,
                            R.color.verdnatura_dark_mint_light_precontrolled
                        )
                    )
                    itemArticleRowSemaforoCon.setBackgroundColor(
                        getColor(
                            context!!,
                            R.color.verdnatura_dark_mint_light_precontrolled
                        )
                    )
                } else if (sale.isPreviousPrepared == "1") {
                    contentLayout.setBackgroundColor(
                        getColor(
                            context!!,
                            R.color.verdnatura_dark_sky_blue
                        )
                    )
                } else {
                    contentLayout.setBackgroundColor(getColor(context!!, R.color.verdnatura_black))
                }


                //sergio: para de momento que pinten las lineas sin actualizar ni nada
                if (sale.isControlled == "2") {
                    itemArticleRowSemaforoCon.setBackgroundColor(
                        getColor(
                            context!!,
                            R.color.verdnatura_white
                        )
                    )
                    contentLayout.setBackgroundColor(
                        getColor(
                            context!!,
                            R.color.verdnatura_dark_mint_light
                        )
                    )
                }


                //ASIGNAMOS VALOR A LA VSITA
                this.sale = sale

            }
        }
    }

    private fun paintTicketcolor(color: String, itemTicketColor: View) {

        when (color) {
            "ROJO" -> itemTicketColor.setBackgroundColor(Color.RED)
            "AMARILLO" -> itemTicketColor.setBackgroundColor(Color.YELLOW)
            "VERDE" -> itemTicketColor.setBackgroundColor(Color.GREEN)
            "AZUL" -> itemTicketColor.setBackgroundColor(Color.BLUE)
            "BLANCO" -> itemTicketColor.setBackgroundColor(Color.WHITE)
            // "NEGRO"-> itemTicketColor.setBackgroundResource((R.drawable.rectangle))

            "NEGRO" -> {
                itemTicketColor.setBackgroundColor(Color.BLACK)
                itemTicketColor.setBackgroundResource((R.drawable.border))
            }
            "VIOLETA" -> itemTicketColor.setBackgroundColor(0xFF4c2882.toInt())
            "GRIS" -> itemTicketColor.setBackgroundColor(Color.GRAY)
            "ORANGE" -> itemTicketColor.setBackgroundColor(
                getColor(
                    context!!,
                    android.R.color.holo_orange_light
                )
            )
            "MORADO" -> itemTicketColor.setBackgroundColor(
                getColor(
                    context!!,
                    android.R.color.holo_purple
                )
            )
            "CELESTE" -> itemTicketColor.setBackgroundColor(0xFF5b85F5.toInt())
            "ROSA" -> itemTicketColor.setBackgroundColor(0xFFfc0fc0.toInt())
            "DORADO" -> itemTicketColor.setBackgroundColor(0xFFb8860b.toInt())
            "MARRON" -> itemTicketColor.setBackgroundColor(0xFF663300.toInt())
            "PERLA" -> itemTicketColor.setBackgroundColor(0xFFcdcecf.toInt())
            "MARFIL" -> itemTicketColor.setBackgroundColor(0xFFf7eada.toInt())
            "CIAN" -> itemTicketColor.setBackgroundColor(0xFF00ffff.toInt())
            "BEIGE" -> itemTicketColor.setBackgroundColor(0xFFf2e7bf.toInt())


            else -> {

            }
        }
        // itemTicketColor.setBackgroundResource((R.drawable.rectangle))


    }
}