package es.verdnatura.presentation.view.feature.collection.fragment


import android.content.Context
import android.content.Intent
import android.graphics.drawable.Drawable
import android.media.MediaPlayer
import android.net.Uri
import android.os.Build
import android.os.Bundle
import android.speech.SpeechRecognizer
import android.util.Log
import android.view.View
import android.view.View.GONE
import android.view.View.VISIBLE
import android.view.inputmethod.EditorInfo
import android.view.inputmethod.InputMethodManager
import android.widget.ImageView
import android.widget.Toast
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import es.verdnatura.R
import es.verdnatura.databinding.FragmentCollectionBinding
import es.verdnatura.domain.ConstAndValues.CONTROLADOR
import es.verdnatura.domain.ConstAndValues.OK
import es.verdnatura.domain.ConstAndValues.ON_CHECKING
import es.verdnatura.domain.ConstAndValues.ON_PREPARATION
import es.verdnatura.domain.ConstAndValues.PRECHECKER
import es.verdnatura.domain.ConstAndValues.SACADOR
import es.verdnatura.domain.toast
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.*
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.component.CustomDialogInput
import es.verdnatura.presentation.view.component.CustomDialogList
import es.verdnatura.presentation.view.component.CustomDialogThreeButtons
import es.verdnatura.presentation.view.feature.articulo.adapter.BarcodeAdapter
import es.verdnatura.presentation.view.feature.articulo.model.BarcodeVO
import es.verdnatura.presentation.view.feature.collection.ItemVO
import es.verdnatura.presentation.view.feature.collection.adapter.SaleAdapter
import es.verdnatura.presentation.view.feature.main.activity.MainActivity
import es.verdnatura.presentation.view.feature.pasillero.model.PasillerosItemVO
import es.verdnatura.presentation.view.feature.sacador.model.*
import timber.log.Timber
import timber.log.Timber.d


class CollectionFragment(
    var collection: CollectionVO = CollectionVO(),
    var type: String = SACADOR
) : BaseFragment<FragmentCollectionBinding, CollectionViewModel>(CollectionViewModel::class) {

    private lateinit var dataMessageSalix: DataMessageSalix
    private var sales: List<SaleVO> = listOf()
    private var saleAdapter: SaleAdapter? = null
    private var lm: LinearLayoutManager? = null
    private var storedPosition: Int = 0
    private var pasillerosItemClickListener: OnPasillerosItemClickListener? = null
    private lateinit var customDialogList: CustomDialogList
    private var placementSupplyAdapter: BarcodeAdapter? = null
    private var listPlacementSupply: ArrayList<BarcodeVO> = ArrayList()
    private var storedShelvingPosition: Int = 0
    private lateinit var customDialogInput: CustomDialogInput
    private lateinit var customDialogThreeButtons: CustomDialogThreeButtons
    private var ticketSelected: String = ""
    private lateinit var customDialog: CustomDialog
    private var goBack: Boolean = false
    private var goBack2: Boolean = false
    var mperror: MediaPlayer? = null
    var mpok: MediaPlayer? = null
    private var itemShelvingFkStored: String = ""
    private var storedBackPosition: Int = 0
    private var tickets: ArrayList<String> = ArrayList()
    private var mistakeSale: SaleVO? = null
    private var positionIncreaseQuantity = 0
    private var positionReject = 0
    private var quantityIncrease = ""
    private var quantityReject = ""
    private var typeCollectionMissing = ""
    private var positionCollectionMissing = 0
    private var quantityCollectionMissing = ""
    private var isMarking = false
    private var positionCollectionSplit = 0
    private var quantityCollectionSplit = ""
    //private var myKM: KeyguardManager? = null

    private var state = 0

    private var mIsListening = false
    private var placementPicked: PlacementVO? = null

    private var lastPlacementPicked: PlacementVO? = null

    private var originalItemScan: String = ""
    private var workerFkFromTicket = ""

    companion object {
        fun newInstance(collection: CollectionVO, type: String) =
            CollectionFragment(collection, type)
    }

    override fun onAttach(context: Context) {
        if (context is OnPasillerosItemClickListener) pasillerosItemClickListener = context
        super.onAttach(context)
    }

    override fun getLayoutId(): Int = R.layout.fragment_collection

    override fun onCreate(savedInstanceState: Bundle?) {

        mperror = MediaPlayer.create((activity as MainActivity), R.raw.error)
        mpok = MediaPlayer.create((activity as MainActivity), R.raw.ok)

        if ((collection.collectionFk != null) && type.equals(CONTROLADOR)) {

            //sergio: comprueba si viene de la pantalla de controlador
            viewModel.ticket_checkFullyControlled(
                getData(USER),
                getData(PASSWORD),
                collection.collectionFk,
                "ticket_checkFullyControlled"
            )
            workerFkFromTicket = collection.tickets.get(0).sales.get(0).workerFk
        }

        if (collection.tickets.isEmpty()) {
            viewModel.collectionTicketGet(
                getData(USER),
                getData(PASSWORD),
                collection.collectionFk,
                getData(SECTORFK),
                type,
                "collection_getTickets"
            )
        }


        super.onCreate(savedInstanceState)
    }

    override fun onResume() {
        super.onResume()
        scanRequest()
    }

    override fun init() {
        customDialogList = CustomDialogList(requireContext())
        customDialogInput = CustomDialogInput(requireContext())
        customDialog = CustomDialog(requireContext())
        customDialogThreeButtons = CustomDialogThreeButtons(requireContext())
        ma.hideBottomNavigation(View.GONE)
        if (binding.splashProgress != null) binding.splashProgress.visibility = VISIBLE

        setEvents()
        setToolBar()

        if (type == SACADOR && getData(VOZ) != "NO") {
            setSpeak()
        }

        if (collection.tickets.isNotEmpty()) {
            createCollectionList()
        }

        super.init()
    }

    override fun onPause() {
        super.onPause()
        goBack = true
        goBack2 = true
    }

    private fun setToolBar() {
        binding.mainToolbar.toolbarSubtitle.visibility = View.VISIBLE
        if (!collection.collectionFk.isNullOrEmpty()) {
            binding.mainToolbar.toolbarTitle.text = collection.collectionFk
        }


        val listIcons: ArrayList<ImageView> = ArrayList()
        val iconPrint = ImageView(context)
        iconPrint.setImageResource(R.drawable.ic_print_black_24dp)
        val iconAdd = ImageView(context)
        iconAdd.setImageResource(R.drawable.ic_playlist_add_black_24dp)
        val iconFace = ImageView(context)
        iconFace.setImageResource(R.drawable.face_man_outline)
        val iconPhone = ImageView(context)
        iconPhone.setImageResource(R.drawable.phone_call)

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {

            iconPrint.tooltipText=getTooltip(R.drawable.ic_local_parking_black_24dp)
            iconAdd.tooltipText=getTooltip(R.drawable.ic_playlist_add_black_24dp)
            iconFace.tooltipText=getTooltip(R.drawable.face_man_outline)
            iconPhone.tooltipText=getTooltip(R.drawable.phone_call)
        }


        if (type.equals("CHECKER")){
            listIcons.add(iconPhone)
                }

        listIcons.add(iconFace)
        listIcons.add(iconPrint)
        listIcons.add(iconAdd)
        //listIcons.add(iconSearch)


        binding.mainToolbar.toolbarIcons.adapter =
            ToolBarAdapterTooltip(listIcons, object : OnOptionsSelectedListener {
                override fun onOptionsItemSelected(item: Drawable) {

                    when (item){
                        iconPrint.drawable->print()
                        iconAdd.drawable->addItem()
                        iconFace.drawable->showUser()
                        iconPhone.drawable->getExtensionFromUser()
                    }

                }
            })
        binding.mainToolbar.toolbarIcons.layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.HORIZONTAL, false)
    }

    private fun getExtensionFromUser() {


        binding.splashProgress.visibility=VISIBLE
        viewModel.sip_getExtension(getData(USER),
            getData(PASSWORD), workerFkFromTicket,"sip_getExtension")


    }

    private fun scanRequest() {
        if (binding.scanInput != null) {
            binding.scanInput.requestFocus()
        }
        hideKeyboards()
    }

    private fun showUser() {

        ma.onPasillerosItemClickListener(
            PasillerosItemVO(title = "Control de usuario de vehículo"),
            workerFkFromTicket
        )

    }

    private fun setEvents() {

        binding.mainToolbar.backButton.setOnClickListener {
            requireActivity().onBackPressed()
        }
        //ESCANER =========
        hideKeyboards()
        /* if (type.equals(CONTROLADOR)){
             binding.scanInput.inputType= InputType.TYPE_CLASS_NUMBER
         }*/
        binding.scanInput.requestFocus()
        binding.scanInput.setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5) {//ID=0 ACTION_NEXT  ID=5 ACTION_UNESPECEFIED)
                if (!binding.scanInput.text.toString().isNullOrEmpty()) {


                    //sergio:para ver si ha marcado dos veces el mismo ticket
                    binding.scanInput.setText(textScanned_filterDouble(binding.scanInput.text!!.toString()))

                    findSale(binding.scanInput.text.toString())

                }
                binding.scanInput.setText("")
                hideKeyboards()
                return@setOnEditorActionListener true
            }
            true
        }

        hideKeyboards()

        //LISTA =========
        binding.collectionSwipe.setOnRefreshListener {

            isMarking = false
            binding.collectionSwipe.isRefreshing = false
           // if (type != PRECHECKER) {
                binding.splashProgress.visibility = VISIBLE
                viewModel.collectionTicketGet(
                    getData(USER),
                    getData(PASSWORD),
                    collection.collectionFk,
                    getData(SECTORFK),
                    type, "collection_getTickets"
                )
                binding.collectionSwipe.isRefreshing = false
            //}
        }
    }

    private fun hideKeyboards() {
        try {
            requireActivity().hideKeyboard()
        } catch (e: Exception) {
        }
    }

    private fun showKeyboards() {
        try {
            requireActivity().showKeyboard()
        } catch (e: Exception) {
        }
    }

    private fun showKeyboardsIN() {
        try {
            requireActivity().showKeyboardIn()
        } catch (e: Exception) {
        }
    }

    override fun observeViewModel() {
        with(viewModel) {
            binding.splashProgress.visibility = GONE
            collectionTicketList.observe(viewLifecycleOwner, Observer {

                if (!it.isError) {
                    if (it.tickets.isNotEmpty()) {
                        collection = it
                        createCollectionList()
                        workerFkFromTicket = it.tickets.get(0).sales.get(0).workerFk

                    } else {
                        binding.mainToolbar.toolbarSubtitle.text = "0/0"
                    }
                } else {
                    customDialog.setTitle("Error").setDescription(it.errorMessage)
                        .setOkButton("Aceptar") {
                            customDialog.dismiss()
                            if (activity != null) requireActivity().onBackPressed()
                        }.show()
                }

            })

            responseUsedShelves.observe(viewLifecycleOwner, Observer {
                binding.splashProgress.visibility = GONE

                if (!goBack){
                ma.messageWithSound(
                    if (it.isError) it.errorMessage else "Baldas guardadas",
                    it.isError,true
                )}

            })
            responseItem_updatePackingShelve.observe(viewLifecycleOwner, Observer {
                binding.splashProgress.visibility = GONE
                ma.messageWithSound(
                    if (it.isError) it.errorMessage else "Packing guardado",
                    it.isError,!it.isError
                )
                if (!it.isError) {
                    viewModel.collectionTicketGet(
                        getData(USER),
                        getData(PASSWORD),
                        collection.collectionFk,
                        getData(SECTORFK),
                        type,
                        "collection_getTickets"
                    )
                }

            })

            responseSaleGroup.observe(viewLifecycleOwner, Observer {

                if (it.isError) {

                    ma.messageWithSound(it.errorMessage, true,true)
                } else {
                    ma.messageWithSound("Previa recogida", false,true)
                    binding.splashProgress.visibility = VISIBLE
                    viewModel.collectionTicketGet(
                        getData(USER),
                        getData(PASSWORD),
                        collection.collectionFk,
                        getData(SECTORFK),
                        type,
                        "collection_getTickets"
                    )
                }
            })
            placementSuppleyList.observe(viewLifecycleOwner, Observer {
                if (binding.splashProgress != null) binding.splashProgress.visibility = View.GONE
                if (!goBack) printShelvingResult(it)
                goBack = false
            })

            item.observe(viewLifecycleOwner, Observer {
                if (binding.splashProgress != null) binding.splashProgress.visibility = View.GONE
                if (!goBack) toastDisponibility(it)
                goBack = false
            })

            responseNew.observe(viewLifecycleOwner, Observer {
                if (binding.splashProgress != null) binding.splashProgress.visibility = View.GONE
                // if (!goBack) {
                if (it.isError) {
                    customDialog.setTitle("Disponibilidad").setDescription(it.errorMessage)
                        .setKoButton("Cerrar") {
                            scanRequest()
                            customDialog.dismiss()
                        }.show()
                } else {

                    if (binding.splashProgress != null) binding.splashProgress.visibility =
                        View.VISIBLE


                    if (!goBack){
                    sendSalixMessage(
                        item = dataMessageSalix.item,
                        ticketFk = dataMessageSalix.ticket,
                        quantity = dataMessageSalix.quantity,
                        salePerson = dataMessageSalix.personSale
                    )
                    viewModel.collectionTicketGet(
                        getData(USER),
                        getData(PASSWORD),
                        collection.collectionFk,
                        getData(SECTORFK),
                        type, "collection_getTickets"
                    )

                    //  //Log.i("VERDNATURA:","La collection es ${collection.collectionFk}")
                }
                }
                // }

                goBack = false

            })

            responseCode.observe(viewLifecycleOwner, Observer {
                if (binding.splashProgress != null) binding.splashProgress.visibility = View.GONE
                if (!goBack2) {
                    if (it.isError) {
                        customDialog.setTitle("Error").setDescription(it.errorMessage)
                            .setKoButton("Cerrar") {
                                scanRequest()
                                customDialog.dismiss()
                            }.show()
                    } else {
                        if (checkItemScan(it.response)) {
                            scanRequest()
                            customDialogList.dismiss()
                            mpok?.start()
                            onQuantityOfShelvingSelected(itemShelvingFkStored)
                        } else {
                            customDialogList.setValueTwo("")
                            //showErrorMessage("El resultado del procedimiento barcodeToItem de la etiqueta escaneada es: " +it.response)
                            showErrorMessage("La línea de compra de la etiqueta escaneada no existe.")
                            if (mperror != null) mperror?.start()
                        }
                    }
                }

                goBack2 = false

            })

            responseMissingTrash.observe(viewLifecycleOwner, Observer {

             if (!goBack){
                 if (it.isError) {
                     ma.messageWithSound(it.errorMessage, isError = true,true)

                 } else {

                     missingTrashSuccesful()

                 }
             } //sergio:esto ya estaba
                /* binding.splashProgress.visibility = View.VISIBLE
                     viewModel.collectionTicketGet(user,password,collection.collectionFk,getData(SECTORFK))*/
            })


            responseIncQuantity.observe(viewLifecycleOwner, Observer {

                if (it.isError) {
                    ma.messageWithSound(it.errorMessage, isError = true,true)

                } else {
                    if (!goBack){    incresaseSuccesful()}else{goBack=false}
                }

            })

            responseSaleMove.observe(viewLifecycleOwner, Observer {

                if (it.isError) {
                    ma.messageWithSound(it.errorMessage, isError = true,true)

                } else {
                    if (!goBack)
                   responseSaleMoveSuccesful()
                    goBack=false
                }

            })
            responseGetExtensionFromUserId.observe(viewLifecycleOwner, Observer {

                binding.splashProgress.visibility=GONE

                if (it.isError) {
                    if (!goBack)
                    ma.messageWithSound(it.errorMessage+"Usuario:"+workerFkFromTicket, isError = true,true)
                    goBack = false
                } else {
                    if (!goBack)
                    callPicker(it.response)
                    goBack = false
                }

            })


            mistakeList.observe(viewLifecycleOwner, Observer {
                if (binding.splashProgress != null) binding.splashProgress.visibility = GONE
                /*sergio:se quita y se pone esta comprobacion porque si no aparece de nuevo otra vez al volver atrás */
                if (!goBack)
                showMistakeList(it.list)
                goBack = false
            })
            //sergio: para si hay algun ticket por revisar
            responsecheckfully.observe(viewLifecycleOwner, Observer {
                binding.splashProgress.visibility = View.GONE
                if (it.isError) {
                    ma.messageWithSound(it.errorMessage,it.isError,false)
                    /*customDialog.setTitle("Error").setDescription(it.errorMessage)
                        .setOkButton("Aceptar") {
                            customDialog.dismiss()
                        }.show()*/
                } else {
                    //sergio: devuelve false si no hay para revisar y <> false.
                    if (it.response != "false") {

                        customDialog.setTitle("Aviso")
                            .setDescription("Pendiente de revisar : " + it.response)
                            .setOkButton("Aceptar") {
                                customDialog.dismiss()

                            }
                            .setKoButton("Revisar") {
                                binding.splashProgress.visibility = View.VISIBLE
                                viewModel.collectionTicketGet(
                                    getData(USER),
                                    getData(PASSWORD),
                                    it.response,
                                    getData(SECTORFK),
                                    type,
                                    "collection_getTickets"
                                )
                                customDialog.dismiss()
                            }
                        customDialog.show()
                    }
                    // if (!goBack)navigateToCollectionList(it)
                    //goBack = false
                }
            })

            responseSalixMessage.observe(viewLifecycleOwner, Observer {


                if (it.isError) {

                    if (!goBack){
                        ma.messageWithSound(it.errorMessage,it.isError,false)}else{goBack=false}

                    /*customDialog.setTitle("Error").setDescription(it.errorMessage)
                        .setOkButton("Aceptar") {
                            customDialog.dismiss()
                        }.show()*/
                }
            })


        }
    }

    private fun responseSaleMoveSuccesful() {
        sales[positionCollectionSplit].quantity = quantityCollectionSplit

        if (quantityCollectionSplit == "0")
            markLine(positionCollectionSplit, type)
        saleAdapter!!.notifyDataSetChanged()

        val ticket =
            "[" + sales[positionCollectionSplit].ticketFk + "](https://salix.verdnatura.es/#!/ticket/" + sales[positionCollectionSplit].ticketFk + "/summary)"
        val message =
            "Se ha enviado a Split el articulo " + sales[positionCollectionSplit].itemFk + " del ticket " + ticket
        sendSalixMessageNew(message,sales[positionCollectionSplit].salePersonFk)

    }

    private fun callPicker(extensionNumber:String) {


                try {
                    val intent = Intent(Intent.ACTION_DIAL).apply {
                        data = Uri.parse("tel:$extensionNumber")
                    }

                    startActivity(intent)
                }catch (e:Exception){
                    d("VERDNATURA::"+e.message)
                }


    }

    private fun getSalePerson(ticketFk: String): String {

        var salePerson = ""
        for (it in sales) {
            if (it.ticketFk.equals(ticketFk)) {
                salePerson = it.salePersonFk
                break
            }
        }
        return salePerson
    }

    private fun sendSalixMessage(
        item: String,
        ticketFk: String,
        quantity: String,
        salePerson: String
    ) {
        val ticket =
            "[" + ticketFk + "](https://salix.verdnatura.es/#!/ticket/" + ticketFk + "/summary)"
        val message =
            "Se ha añadido el articulo " + item + " con " + quantity + " unidad/es al ticket " + ticket


        /*    d("VERDNATURA:","el comercial es " + salePerson)
            d("VERDNATURA:","el item  es " + item)
            d("VERDNATURA:","el ticket es es " + ticketFk)
            d("VERDNATURA:","la cantidad es " + quantity)*/

        viewModel.sendChekingPresence(
            token = getData(TOKEN),
            workerId = salePerson,
            message = message, "sendChekingPresence"
        )
    }

    //CREATE LIST
    private fun createCollectionList() {
        state = 0
        if (type == SACADOR && getData(VOZ) != "NO") {
            // initialize()
            speak("Colección cargada, diga listo para empezar.")
        }

        binding.mainToolbar.toolbarTitle.text = collection.collectionFk


        if (binding.splashProgress != null) binding.splashProgress.visibility = View.GONE
        var salesList: ArrayList<SaleVO> = ArrayList()
        tickets = ArrayList()
        var observations = ""
        collection.tickets.forEach { ticket ->
            ticket.sales.forEach { saleVO ->
                if (type == SACADOR && saleVO.quantity != "0") {
                    salesList.add(saleVO)
                    if (tickets.firstOrNull { it == saleVO.ticketFk }.isNullOrEmpty())
                        tickets.add(saleVO.ticketFk)
                } else if (type == CONTROLADOR || type == PRECHECKER) {
                    //sergio el type CHECKED =CONTROLADOR
                    salesList.add(saleVO)
                    if (tickets.firstOrNull { it == saleVO.ticketFk }.isNullOrEmpty())
                        tickets.add(saleVO.ticketFk)
                }
            }

            observations = observations + " " + ticket.observations
        }


        sales = salesList.sortedWith(compareBy({ it.saleOrder }))

        //pruebas sergio
        // sales = salesList.sortedWith(compareBy({it.placements.get(0).created}))
        //sales = salesList.sortedWith(compareBy({it.placements.get(0).placement}))
        // SalesSorter()


        saleAdapter =
            SaleAdapter(sales, pasillerosItemClickListener!!, object : OnQuantityClickListener {

                override fun onQuantityClick(sale: SaleVO) {

                    sales.forEachIndexed { index, saleVO ->
                        if (saleVO.saleFk == sale.saleFk) {
                            if (type != "PRECHECKER") {
                                showQuantityDialog(index)
                            }

                        }
                    }

                }
            }, object : OnSaleClickListener {
                override fun onSaleClick(sale: SaleVO) {

                    sales.forEachIndexed { index, saleVO ->
                        if (saleVO.saleFk == sale.saleFk) {

                            if (type == CONTROLADOR || type == PRECHECKER) {// sergio:cuidado , esta es de lo último para marcar colores
                                markLine(index, type)
                                //////Log.i("VERDNATURA:","marcamos linea")
                            } else if (type == SACADOR) {

                                if (sale.isPrepared == "0" || sale.isPrepared.isNullOrEmpty()) {
                                    showScanner(index, sale)
                                } else {
                                    unMarkLine(index, OK)
                                }

                            }
                        }

                    }
                }

            }, object : OnMistakeClickListener {
                override fun onMistakeClickListener(sale: SaleVO) {
                    if (type != "PRECHECKER") {
                        if (binding.splashProgress != null) binding.splashProgress.visibility =
                            View.VISIBLE
                        mistakeSale = sale
                        viewModel.mistakeType(usuario = getData(USER),
                            getData(PASSWORD))
                    }
                }
            }, object : onPackingClickListener {
                override fun onPackingClick(sale: SaleVO) {
                    item_updatePackingShelve(sale.itemFk)
                }
            }
            )

        lm = LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)
        binding.fragmentSacadorCollections.adapter = saleAdapter
        binding.fragmentSacadorCollections.layoutManager = lm
        setTotalLines()

        if (storedPosition != 0)
            setListPosition(storedPosition, true)
        else if (storedBackPosition != 0)
            setListPosition(storedBackPosition, true)

        setScrollListener(lm!!)
        //sergio: se quita porque se realiza desde triggers
        // changeInitTicketState()

        printObservations(observations)
    }

    private fun setScrollListener(lm: LinearLayoutManager) {
        binding.fragmentSacadorCollections.addOnScrollListener(object :
            RecyclerView.OnScrollListener() {
            override fun onScrolled(recyclerView: RecyclerView, dx: Int, dy: Int) {
                storedBackPosition = lm.findFirstVisibleItemPosition()
                super.onScrolled(recyclerView, dx, dy)
            }
        })
    }

    private fun printObservations(observations: String) {
        if (observations.trim().isNotEmpty()) {
            customDialog.setTitle("Observaciones").setDescription(observations)
                .setOkButton("Aceptar") {
                    customDialog.hide()
                    customDialog.dismiss()
                }.show()
        }
    }

    //SEARCH AND MARK
    private fun findSale(txtscan: String) {
        goBack = false
        var index = 0
        var isBreak = false
        var isOk = false
        goBack = false
        goBack2 = false

        if (type == SACADOR) {
            for (saleVO in sales) {
                //sergio: pita si encuentra ticket o el saleGroupFk
                //Log.i("VERDNATURA:","SE busca en sales ${saleVO.ticketFk}")
                //canviar el titcket per salegroup i enviar a pako saleTracking_add (saleGroupFk) i refrescar pantalla.
                d("El salegroup es " + saleVO.saleGroupFk)
                if (saleVO.saleGroupFk == txtscan) {
                    // if (saleVO.ticketFk == txtscan ) {
                    isOk = true
                    viewModel.saleTracking_add(
                        getData(USER),
                        getData(PASSWORD),
                        txtscan,
                        "saleTracking_add"
                    )
                    if ((isOk) && mpok != null) {
                        mpok!!.start()
                    }

                    break

                }

                if (saleVO.isPrepared != "1" && saleVO.isPreviousPrepared != "1") {
                    //1- Por itemFk
                    /* if (txtscan == saleVO.itemFk){
                         mpok!!.start()
                         isOk = true
                         markLine(index,type)
                         break
                     }*/
                    //2- Por carro
                    if (saleVO.placements != null) {
                        var shelvingIndex = 0
                        for (placementVO in saleVO.placements) {
                            // para comprobar si es igual a una matricula
                            if (placementVO.shelving != null && placementVO.visible != null && txtscan.uppercase() == placementVO.shelving.uppercase() && placementVO.visible != "(0)") {
                                if (mpok != null) mpok!!.start()
                                isOk = true
                                //////Log.i("VERDNATURA:","encontrado producto")
                                showShelving(index, shelvingIndex)
                                isBreak = true
                                break
                            }
                            shelvingIndex += 1
                        }
                        if (isBreak) break
                    }

                    //3- Por barcode
                    /*saleVO.Barcodes.forEach { barcode ->
                        if (txtscan == barcode){
                            mpok!!.start()
                            isOk = true
                            markLine(index,type)
                            isBreak = true
                        }
                    }
                    if (isBreak) break*/
                }
                index += 1
            }
            if (!isOk) {
                if (txtscan.contains("-")) {
                    if (tickets.size > 0) {
                        viewModel.parking(
                            getData(USER),
                            getData(PASSWORD),
                            ticketFk = tickets[0],
                            parking = txtscan,
                            "shelvingPark"
                        )
                    }
                    Timber.i("La coleccion es " + tickets[0])
                    if (mpok != null) mpok!!.start()
                    "Ticket aparcado".toast(requireContext())
                } else {
                    if (mperror != null) mperror!!.start()
                }

            }
        } else if (type == CONTROLADOR || type == PRECHECKER) {
            //////Log.i("VERDNATURA:","El tipo es controlador")
            for (saleVO in sales) {
                if (saleVO.isControlled == "0") {
                    //1- Por itemFk
                    if (txtscan == saleVO.itemFk) {
                        if (mpok != null) mpok!!.start()
                        isOk = true
                        markLine(index, type)
                        break
                    }
                    //2- Por barcode
                    saleVO.Barcodes.forEach { barcode ->
                        if (txtscan == barcode) {
                            if (mpok != null) mpok!!.start()
                            isOk = true
                            markLine(index, type)
                            isBreak = true
                        }
                    }
                    if (isBreak) break
                }
                index += 1
            }
            //Lo escaneado no encontrado miramos si es parking o ticket
            if (!isOk) {
                if (txtscan.contains("-")) {
                    if (tickets.size > 0) {
                        viewModel.parking(
                            getData(USER),
                            getData(PASSWORD),
                            ticketFk = tickets[0],
                            parking = txtscan,
                            "shelvingPark"
                        )
                    }
                    if (mpok != null) mpok!!.start()
                    getString(R.string.Ticketaparcado).toast(requireContext())
                } else {
                    if (binding.splashProgress != null) binding.splashProgress.visibility =
                        View.VISIBLE
                    storedPosition = 0
                    storedBackPosition = 0
                  //  if (type != PRECHECKER) {
                        isMarking = false
                        viewModel.collectionTicketGet(
                            getData(USER),
                            getData(PASSWORD),
                            txtscan,
                            getData(SECTORFK),
                            type,
                            "collection_getTickets"
                        )

                        if (type.equals(CONTROLADOR)) {
                            viewModel.ticket_checkFullyControlled(
                                getData(USER),
                                getData(PASSWORD),
                                txtscan,
                                "ticket_checkFullyControlled"
                            )
                        }
                  //  } else {
                    //    binding.splashProgress.visibility = View.GONE
                    //}
                    ////Log.i("VERDNATURA:","La collection es ${txtscan}")
                }
            }
        }

    }

    private fun findSale(txtscan: String, position: Int) {
        var isOk = false
        if (type == SACADOR) {
            var saleVO = sales[position]

            if (saleVO.isPrepared != "1" && saleVO.isPreviousPrepared != "1") {
                //1- Por itemFk
                /*if (txtscan == saleVO.itemFk){
                    if (mpok != null) mpok!!.start()
                    isOk = true
                    markLine(position,type)

                }*/
                if (!isOk) {
                    //2- Por carro
                    var shelvingIndex = 0
                    for (placementVO in saleVO.placements) {
                        if (txtscan.uppercase() == placementVO.shelving.uppercase()) {
                            if (mpok != null) mpok!!.start()
                            isOk = true
                            showShelving(position, shelvingIndex)
                            break
                        }
                        shelvingIndex += 1
                    }
                }
                /*
                if (!isOk){
                    //3- Por barcode
                    saleVO.Barcodes.forEach { barcode ->
                        if (txtscan == barcode){
                            if (mpok != null) mpok!!.start()
                            isOk = true
                            markLine(position,type)
                        }
                    }
                }*/
            }
            if (!isOk) {
                if (txtscan.contains("-")) {
                    tickets.forEach {
                        viewModel.parking(
                            getData(USER),
                            getData(PASSWORD),
                            ticketFk = it,
                            parking = txtscan,
                            "shelvingPark"
                        )
                    }
                    if (mpok != null) mpok!!.start()
                    "Ticket aparcado".toast(requireContext())

                } else {
                    if (mperror != null) mperror!!.start()
                }

            }
        }

    }


    private fun markLine(position: Int, newType: String) {
        state = 0
        //////Log.i("VERDNATURA:","Marcamos linea y enviamos saleTracking")
        if (type == SACADOR) {
            sales[position].isPrepared = if (sales[position].isPrepared == "1") "0" else "1"
            if (sales[position].isPrepared == "1") {
                sales[position].pickedQuantity = sales[position].quantity
            }
        } else if (type == CONTROLADOR)
            sales[position].isControlled = if (sales[position].isControlled == "1") "0" else "1"

        if (type == PRECHECKER) {
            sales[position].isPreControlled= if ( sales[position].isPreControlled == "1") "0" else "1"
           /* sales[position].isControlled =
                    //if (sales[position].isControlled == "2") "0" else "2" //sergio:  para diferenciar precontrol en previa
                if (sales[position].isControlled == "1") "0" else "1"//sergio:  para diferenciar precontrol en previa*/
        }

        saleAdapter!!.notifyDataSetChanged()
        setListPosition(position, false)

       /* if (type != PRECHECKER) {
            saleTrackingReplace(position, newType)
            isMarking = true
        }
*/
        // sergio
       /* if (type == PRECHECKER) {
            saleTrackingReplace(position, newType)
        }*/

        saleTrackingReplace(position, newType)
        isMarking = true
        setTotalLines()
    }

    private fun setListPosition(position: Int, isFromBack: Boolean) {
        storedPosition = position
        if (type == SACADOR) {
            if (binding.fragmentSacadorCollections != null) {
                binding.fragmentSacadorCollections.addViewObserver {
                    lm!!.scrollToPositionWithOffset(position, 0)
                }
            }

        } else if (isFromBack) {
            if (binding.fragmentSacadorCollections != null) {
                binding.fragmentSacadorCollections.addViewObserver {
                    lm!!.scrollToPositionWithOffset(position, 0)
                }
            }
        }

    }

    private fun saleTrackingReplace(position: Int, type: String) {
        //////Log.i("VERDNATURA:","POSICION $position y el tipo $type")
        //////Log.i("VERDNATURA:","El buyfk que no es real es $originalItemScan")
        viewModel.saleTrackingReplace(
            getData(USER),
            getData(PASSWORD),
            saleFk = sales[position].saleFk,
            vIsChecked = if (type == SACADOR) sales[position].isPrepared else if (type == CONTROLADOR ) sales[position].isControlled  else if (type=="PRECHECKER") sales[position].isPreControlled else "1",
            vOriginalQuantity = sales[position].pickedQuantity,
            vStateFk = if (type == CONTROLADOR) "CHECKED" else if (type == "PRECHECKER") "PREVIOUS_CONTROL" else type,//type.toString(),
            vBuyFk = originalItemScan,
            callFunction = "saleTrackingReplace"
        )

    }

    private fun unMarkLine(position: Int, newType: String) {
        state = 0
        if (sales[position].isPrepared == "1") {
            customDialog.setTitle("Desmarcar linea")
                .setDescription("Vas a desmarcar la linea: " + sales[position].itemFk + " ¿Estás seguro?")
                .setOkButton("Desmarcar") {
                    sales[position].isPrepared = "0"
                    sales[position].pickedQuantity = "0"
                    saleAdapter!!.notifyDataSetChanged()
                    setListPosition(position, false)
                    if (type != PRECHECKER) {
                        viewModel.saleTrackingDel(
                            getData(USER),
                            getData(PASSWORD),
                            saleFk = sales[position].saleFk,
                            "saleTracking_del"
                        )
                    }
                    setTotalLines()
                    scanRequest()
                    customDialog.dismiss()
                }.setKoButton("Cancelar") {
                    scanRequest()
                    customDialog.dismiss()
                }.show()
        }

    }

    //SHELVINGS
    private fun showShelving(position: Int, shelvingPosition: Int) {
        storedShelvingPosition = shelvingPosition
        storedPosition = position
        if (binding.splashProgress != null) binding.splashProgress.visibility = View.VISIBLE
        var quantityGet = "0"
        try {
            quantityGet =
                (sales[position].quantity.toInt() - sales[position].pickedQuantity.toInt()).toString()
        } catch (e: Exception) {
        }
        //Log.i("VERDNATURA:","Aparece  para enviar")
        viewModel.itemPlacementSupplyAiming(
            getData(USER),
            getData(PASSWORD),
            itemFk = sales[position].itemFk,
            quantity = quantityGet,
            shelvingFk = sales[position].placements[shelvingPosition].shelving,
            callFunction = "itemPlacementSupplyAiming"
        )
    }

    private fun printShelvingResult(placementSupplyListVO: PlacementSupplyListVO) {
        //////Log.i("VERDNATURA:","Print CustomDialog")
        var shelving = ""
        var item = ""
        var longName = ""
        var total = "0"
        var itemShelvingFk = "0"
        if (!placementSupplyListVO.list.isEmpty()) {
            var placement: PlacementSupplyVO? = null
            placementSupplyListVO.list.forEach { placementVO ->
                if (placementVO.stock != "0" && placement == null)
                    placement = placementVO
            }
            if (placement != null) {
                shelving = placement!!.shelving
                item = placement!!.itemFk
                longName = placement!!.longName
                total = placement!!.total
                itemShelvingFk = placement!!.itemShelvingFk
            }

        }
        listPlacementSupply = ArrayList()
        placementSupplyListVO.list.forEach {
            if (it.stock != "0")
                listPlacementSupply.add(BarcodeVO(code = it.proposal))
            //////Log.i("VERDNATURA:","Array de placementsupply")
        }
        try {
            // Log.i("VERDNATURA:","Pulso COGER-1")
            customDialogList.setTitle("$shelving($item) $total de $longName").setOkButton("Coger") {
                if (customDialogList.getValueTwo().isNotEmpty()) {
                    if (isNumber(customDialogList.getValue()) && isNumber(total) && customDialogList.getValue()
                            .toInt() > total.toInt()
                    ) {
                        "La cantidad supera a la disponible".toast(requireContext())
                    } else if (isNumber(customDialogList.getValue())) {
                        originalItemScan = customDialogList.getValueTwo()
                        // Log.i("VERDATURA","Le pasamos el siguiente item $originalItemScan que es el escaneado")
                        if (checkItemScan(customDialogList.getValueTwo())) {
                            onQuantityOfShelvingSelected(itemShelvingFk)
                            //Log.i("VERDNATURA:","Cantidad seleccionada")
                            mpok?.start()
                            ReviewQuantityForRefreshingAndSorting(
                                customDialogList.getValue().toInt(),
                                sales[storedPosition].quantity.toInt(),
                                sales[storedPosition].pickedQuantity.toInt()
                            )
                            customDialogList.dismiss()
                        } else {
                            itemShelvingFkStored = itemShelvingFk
                            if (binding.splashProgress != null) binding.splashProgress.visibility =
                                View.VISIBLE
                            viewModel.getIdFromCode(usuario = getData(USER),
                        password = getData(PASSWORD),
                                code = customDialogList.getValueTwo(), "barcodeToItem"
                            )
                            customDialogList.dismiss()
                        }
                        scanRequest()
                        hideKeyboards()
                    } else {
                        "cantidad introducida erronea".toast(requireContext())
                    }


                } else {
                    "Escanea item para validar".toast(requireContext())
                }

            }.setKoButton("Cerrar") {
                scanRequest()
                hideKeyboards()
                customDialogList.dismiss()
            }.setHintValue("Cantidad que coges:").setValue(total).setHintValueTwo("Escanea item")
                .setValueTwo("").show()
        } catch (e: Exception) {
            //////Log.i("VERDNATURA:","${e.message.toString()}")
        }
        try {
            customDialogList.getEditTextTwo().post(Runnable {
                customDialogList.getEditTextTwo().requestFocusFromTouch()
                if (activity != null) {
                    val lManager: InputMethodManager =
                        requireActivity().getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
                    lManager.hideSoftInputFromWindow(
                        customDialogList.getEditTextTwo().windowToken,
                        InputMethodManager.SHOW_FORCED
                    )
                }

            })
        } catch (e: Exception) {
        }
        try {
            customDialogList.getEditTextTwo().setOnEditorActionListener { v, actionId, event ->
                if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5) {

                    // Log.i("VERDNATURA:","Comprobando COGER-1")
                    if (customDialogList.getValueTwo().isNotEmpty()) {
                        try {
                            if (isNumber(customDialogList.getValue()) && customDialogList.getValue()
                                    .toInt() > total.toInt()
                            ) {
                                "La cantidad supera a la disponible".toast(requireContext())
                            } else if (isNumber(customDialogList.getValue())) {
                                originalItemScan = customDialogList.getValueTwo()

                                if (checkItemScan(customDialogList.getValueTwo())) {
                                    onQuantityOfShelvingSelected(itemShelvingFk)
                                    mpok?.start()
                                    ReviewQuantityForRefreshingAndSorting(
                                        customDialogList.getValue().toInt(),
                                        sales[storedPosition].quantity.toInt(),
                                        sales[storedPosition].pickedQuantity.toInt()
                                    )
                                    //  SalesSorter(sales)

                                    customDialogList.dismiss()
                                } else {
                                    itemShelvingFkStored = itemShelvingFk
                                    binding.splashProgress.visibility = View.VISIBLE
                                    viewModel.getIdFromCode(
                                        getData(USER),
                                        getData(PASSWORD),
                                        code = customDialogList.getValueTwo(), "barcodeToItem"
                                    )
                                    customDialogList.dismiss()
                                    scanRequest()
                                }
                            } else {
                                "cantidad introducida erronea".toast(requireContext())
                            }

                        } catch (e: Exception) {

                        }

                    } else {
                        "Escanea item para validar".toast(requireContext())
                    }
                    scanRequest()
                    hideKeyboards()
                    return@setOnEditorActionListener true

                }
                false
            }
        } catch (e: Exception) {
        }

        placementSupplyAdapter =
            BarcodeAdapter(listPlacementSupply, object : OnBarcodeRowClickListener {
                override fun onBarcodeRowClickListener(item: BarcodeVO) {
                    placementSupplyListVO.list.forEach {
                        if (it.proposal == item.code) {
                            customDialogList.setValue(it.total)
                            total = it.total
                            itemShelvingFk = it.itemShelvingFk
                        }
                    }

                }
            }, showDelete = false)

        customDialogList.getRecyclerView().adapter = placementSupplyAdapter

        customDialogList.getRecyclerView().layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)

    }

    private fun ReviewQuantityForRefreshingAndSorting(
        quantityPicking: Int,
        quantityTotal: Int,
        quantityPicked: Int
    ) {

        /* Log.i("VERDNATURA::","Picking"+quantityPicking)
         Log.i("VERDNATURA::","Total"+quantityTotal)
         Log.i("VERDNATURA::","Now picked"+quantityPicked)*/

        if (quantityPicked < quantityTotal) {

            //  SalesSorter()
            // Log.i("VERDNATURA::","Lanzamos collection para que refresque adaptador y ordene")
            viewModel.collectionTicketGet(
                getData(USER),
                getData(PASSWORD),
                collection.collectionFk,
                getData(SECTORFK),
                type,
                "collection_getTickets"
            )

        }

    }



    private fun checkItemScan(valueToCheck: String): Boolean {

        if (storedPosition >= 0 && sales.size > storedPosition) {
            val saleToCheck = sales[storedPosition]

            if (saleToCheck.itemFk == valueToCheck)
                return true
            else {
                saleToCheck.Barcodes.forEach { barcode ->
                    if (barcode == valueToCheck)
                        return true
                }
            }
        }
        return false
    }

    private fun onQuantityOfShelvingSelected(itemShelvingFk: String, quantity: String = "0") {
        //1 - MODIFICAR CANTIDAD DEL CARRO
        try {
            val shelvingVisible =
                sales[storedPosition].placements[storedShelvingPosition].visible.substring(
                    1,
                    sales[storedPosition].placements[storedShelvingPosition].visible.indexOf(")")
                )

            if (quantity == "0") {
                sales[storedPosition].placements[storedShelvingPosition].visible =
                    "(" + (shelvingVisible.toInt() - customDialogList.getValue()
                        .toInt()).toString() + ")"
            } else {
                sales[storedPosition].placements[storedShelvingPosition].visible =
                    "(" + (shelvingVisible.toInt() - quantity.toInt()).toString() + ")"
            }

            viewModel.itemShelvingSaleSupplyAdd(
                getData(USER),
                getData(PASSWORD),
                itemShelvingFk = itemShelvingFk,
                saleFk = sales[storedPosition].saleFk,
                quantity = if (quantity != "0") quantity else customDialogList.getValue(),
                callFunction = "itemShelvingSaleSupplyAdd"
            )

        } catch (e: Exception) {
        }

        //2- MODIFICAR EL PICKED DEL SALE
        try {
            if (quantity == "0") {
                sales[storedPosition].pickedQuantity =
                    (sales[storedPosition].pickedQuantity.toInt() + customDialogList.getValue()
                        .toInt()).toString()
            } else {
                sales[storedPosition].pickedQuantity =
                    (sales[storedPosition].pickedQuantity.toInt() + quantity
                        .toInt()).toString()
            }
        } catch (e: Exception) {
        }

        //3- MARCAR LINEA
        if (sales[storedPosition].pickedQuantity != sales[storedPosition].quantity) {
            sales[storedPosition].isPrepared = "1"
            markLine(storedPosition, OK)
        } else {
            markLine(storedPosition, type)
        }

    }

    //OPTIONS
    private fun print() {
        viewModel.collectionStickerPrint(
            getData(USER),
            getData(PASSWORD),
            collectionFk = collection.collectionFk,
            sectorFk = getData(SECTORFK),
            "collectionStickerPrint"
        )


        (getString(R.string.Imprimiendo) + " por " + getData(SECTORDESCRIP)).toast(requireContext())
    }

    private fun addItem() {
        listPlacementSupply = ArrayList()
        collection.tickets.forEach {
            listPlacementSupply.add(BarcodeVO(code = it.ticketFk))
        }
        if (listPlacementSupply.size == 1) {
            ticketSelected = listPlacementSupply[0].code!!
        }

        customDialogList.setTitle(getString(R.string.Agregarartículoparaticket) + " " + ticketSelected)
            .setOkButton(getString(R.string.Agregar)) {
                if (ticketSelected.isNullOrEmpty()) {
                    getString(R.string.Seleccionaunticketdelaista).toast(requireContext())
                } else if (customDialogList.getValue()
                        .isNullOrEmpty() || customDialogList.getValueTwo().isNullOrEmpty()
                ) {
                    getString(R.string.Todosloscampossonobligatorios).toast(requireContext())
                } else {
                    if (binding.splashProgress != null) binding.splashProgress.visibility =
                        View.VISIBLE

                    //sergio datos para mensaje salix

                    dataMessageSalix = DataMessageSalix(
                        ticket = ticketSelected,
                        item = customDialogList.getValue(),
                        quantity = customDialogList.getValueTwo(),
                        personSale = getSalePerson(ticketSelected)
                    )

                    viewModel.collectionAddItem(
                        getData(USER),
                        getData(PASSWORD),
                        itemFk = customDialogList.getValue(),
                        ticketFk = ticketSelected,
                        quantityFk = customDialogList.getValueTwo(),
                        warehouseFk = getData(WAREHOUSEFK),
                        callFunction = "collection_addItem"
                    )


                    scanRequest()
                    customDialogList.dismiss()
                }

            }.setKoButton("Cerrar") {
                scanRequest()
                customDialogList.dismiss()
            }.setHintValue(getString(R.string.Artículo)).setValue("")
            .setHintValueTwo(getString(R.string.Cantidad)).setValueTwo("").show()

        customDialogList.getEditText().requestFocus()
        hideKeyboards()

        customDialogList.getEditText().setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5) {
                if (!customDialogList.getValue().isNullOrEmpty()) {
                    customDialogList.getEditTextTwo().requestFocus()
                }
                hideKeyboards()
                return@setOnEditorActionListener true
            }
            false
        }

        customDialogList.getEditTextTwo().setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5) {
                if (!customDialogList.getValueTwo().isNullOrEmpty()) {
                    if (ticketSelected.isNullOrEmpty()) {
                        getString(R.string.Seleccionaunticketdelaista).toast(requireContext())
                    } else if (customDialogList.getValue()
                            .isNullOrEmpty() || customDialogList.getValueTwo().isNullOrEmpty()
                    ) {
                        getString(R.string.Todosloscampossonobligatorios).toast(requireContext())
                    } else {
                        if (binding.splashProgress != null) binding.splashProgress.visibility =
                            View.VISIBLE

                        dataMessageSalix = DataMessageSalix(
                            ticket = ticketSelected,
                            item = customDialogList.getValue(),
                            quantity = customDialogList.getValueTwo(),
                            personSale = getSalePerson(ticketSelected)
                        )

                        viewModel.collectionAddItem(
                            getData(USER),
                            getData(PASSWORD),
                            itemFk = customDialogList.getValue(),
                            ticketFk = ticketSelected,
                            quantityFk = customDialogList.getValueTwo(),
                            warehouseFk = getData(WAREHOUSEFK),
                            callFunction = "collection_addItem"
                        )
                        scanRequest()
                        customDialogList.dismiss()
                    }
                }
                hideKeyboards()
                return@setOnEditorActionListener true
                scanRequest()
                customDialogList.dismiss()
            }
            false
        }

        placementSupplyAdapter =
            BarcodeAdapter(listPlacementSupply, object : OnBarcodeRowClickListener {
                override fun onBarcodeRowClickListener(item: BarcodeVO) {
                    customDialogInput.setTitle(getString(R.string.ArtículoparaTicket) + " " + item.code!!)
                    ticketSelected = item.code!!
                    customDialogList.setTitle(getString(R.string.Agregarartículoparaticket) + " " + ticketSelected)
                }
            }, showDelete = false)

        customDialogList.getRecyclerView().adapter = placementSupplyAdapter

        customDialogList.getRecyclerView().layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)

    }

    /* private fun showDisponibility() {

         customDialogInput.setTitle(getString(R.string.Verdisponible))
             .setDescription(getString(R.string.Escaneaetiqueta))
             .setOkButton(getString(R.string.Buscar)) {
                 try {
                     if (binding.splashProgress != null) {
                         binding.splashProgress.visibility = View.VISIBLE
                     }

                 } catch (e: Exception) {
                 }

                 hideKeyboards()
                 viewModel.itemGetAvailable(
                     usuario = user,
                     password = password,
                     itemFk = customDialogInput.getValue(),
                     warehouseFk = warehouseFk,
                     "item_GetVisibleAvailable"
                 )
                 scanRequest()
                 customDialogInput.dismiss()

             }.setKoButton("Cancelar") {
             if (binding.splashProgress != null) {
                 binding.splashProgress.visibility = View.GONE
             }
             hideKeyboards()
             scanRequest()
             customDialogInput.dismiss()
         }.setValue("").show()
         customDialogInput.getEditText().requestFocus()
         customDialogInput.getEditText().setOnEditorActionListener { v, actionId, event ->
             if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                 if (!customDialogInput.getValue().isNullOrEmpty()) {
                     if (binding.splashProgress != null) binding.splashProgress.visibility = View.VISIBLE
                     viewModel.itemGetAvailable(
                         usuario = user,
                         password = password,
                         itemFk = customDialogInput.getValue(),
                         warehouseFk = warehouseFk
                     ,"item_GetVisibleAvailable")
                 }
                 customDialogInput.setValue("")
                 scanRequest()
                 customDialogInput.dismiss()
                 hideKeyboards()
                 return@setOnEditorActionListener true
             }
             false
         }
     }*/

    private fun toastDisponibility(item: ItemVO) {
        if (item.available.isNullOrEmpty()) {
            item.available = "0"
        }

        customDialog.setTitle("Artículo: " + item.id)
            .setDescription("Disponible: " + item.available).setOkButton("Aceptar") {
                scanRequest()
                customDialog.dismiss()
            }.show()

    }

    private fun setTotalLines() {

        var totalMark = 0
        sales.forEach {
            if (type == SACADOR) {
                if (it.isPrepared == "1" || it.isControlled == "1" || it.isPreviousPrepared == "1") {
                    totalMark += 1
                } else if (it.quantity == "0") {
                    totalMark += 1
                }
            } else if (type == CONTROLADOR || type == PRECHECKER) {
                if (it.isControlled == "1" || it.isControlled == "2")
                    totalMark += 1

            }
        }
        //toolbar_title.text = "" + totalMark + "/" + sales.size
        if (binding.mainToolbar.toolbarTitle != null) binding.mainToolbar.toolbarTitle.text =
            if (collection.collectionFk != null) collection.collectionFk else ""
        if (binding.mainToolbar.toolbarSubtitle != null && totalMark != null && sales != null) binding.mainToolbar.toolbarSubtitle.text =
            "" + totalMark + "/" + sales.size

        if (totalMark == sales.size) {
            getString(R.string.Coleccióncompleta).toast(this.context, Toast.LENGTH_SHORT)
            if (!goBack)
            ticketCollection_setUsedShelves()
            goBack=false

            //showBottomNavigation(View.VISIBLE)
            //sergio: se quita porque se realiza desde triggers

            changeTicketState()
        }
    }

    private fun ticketCollection_setUsedShelves() {


        if (type.equals("CHECKER") && (isMarking  && getData(SECTORDESCRIP).uppercase() != "ALGEMESI V")
        ) {


            customDialogInput.setTitle("Número de baldas del ticket")
                .setDescription("Indica el número de baldas que ocupa el ticket")
                .setOkButton("Aceptar") {
                    if (!customDialogInput.getValue().isNullOrEmpty()) {

                        binding.splashProgress.visibility = VISIBLE
                        viewModel.ticketCollection_setUsedShelves(
                            getData(USER),
                            getData(PASSWORD),
                            ticketFk = collection.collectionFk,
                            usedShelves = customDialogInput.getValue(),
                            "ticketCollection_setUsedShelves"
                        )
                        hideKeyboards()
                        //  showKeyboards()

                    }
                    customDialogInput.setValue("")
                    customDialogInput.dismiss()
                    showKeyboards()
                    //hideKeyboards()
                    //showKeyboards()
                }.setKoButton("Cancelar") {
                    customDialogInput.dismiss()
                    // hideKeyboards()
                    showKeyboards()
                }.setValue("").show()

            customDialogInput.setInputText()
            // customDialogInput.currentFocus
            customDialogInput.setFocusText()
            showKeyboards()

            customDialogInput.getEditText().setOnEditorActionListener { v, actionId, event ->
                if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                    if (!customDialogInput.getValue().isNullOrEmpty()) {

                        binding.splashProgress.visibility = VISIBLE
                        viewModel.ticketCollection_setUsedShelves(
                            getData(USER),
                            getData(PASSWORD),
                            ticketFk = collection.collectionFk,
                            usedShelves = customDialogInput.getValue(),
                            "ticketCollection_setUsedShelves"
                        )
                        hideKeyboards()
                    }
                    customDialogInput.setValue("")
                    scanRequest()
                    customDialogInput.dismiss()
                    hideKeyboards()
                    return@setOnEditorActionListener true
                }
                false
            }


        }


    }

    private fun item_updatePackingShelve(itemFk: String) {


        isMarking = false
        customDialogInput.setTitle("Número de paquetes en balda")
            .setDescription("Indica el número de paquetes que caben en una balda")
            .setOkButton("Aceptar") {
                if (!customDialogInput.getValue().isNullOrEmpty()) {

                    binding.splashProgress.visibility = VISIBLE
                    viewModel.item_updatePackingShelve(
                        getData(USER),
                        getData(PASSWORD),
                        vSelf = itemFk,
                        vPacking = customDialogInput.getValue(),
                        "item_updatePackingShelve"
                    )
                    hideKeyboards()
                    //showKeyboards()

                }
                customDialogInput.setValue("")
                customDialogInput.dismiss()
                // hideKeyboards()
                showKeyboards()
            }.setKoButton("Cancelar") {

                customDialogInput.dismiss()
                //hideKeyboards()
                showKeyboards()
            }.setValue("").show()

        customDialogInput.setInputText()
        customDialogInput.currentFocus
        customDialogInput.setFocusText()
        showKeyboards()

        customDialogInput.getEditText().setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                if (!customDialogInput.getValue().isNullOrEmpty()) {

                    binding.splashProgress.visibility = VISIBLE
                    viewModel.ticketCollection_setUsedShelves(
                        getData(USER),
                        getData(PASSWORD),
                        ticketFk = collection.collectionFk,
                        usedShelves = customDialogInput.getValue(),
                        "ticketCollection_setUsedShelves"
                    )

                }
                customDialogInput.setValue("")
                scanRequest()
                customDialogInput.dismiss()
                hideKeyboards()
                return@setOnEditorActionListener true
            }
            false
        }

    }

    private fun showBottomNavigation(visibilitiy: Int) {
        //requireActivity().main_bottom_navigation.visibility = visibilitiy
    }

    //FALTAS / BASURA / SPLIT
    private fun showQuantityDialog(position: Int) {
        customDialogThreeButtons.setDescription(getString(R.string.txtnuevacantidad)).setValue("")
            .setOkButton(getString(R.string.Faltas)) {
                checkAndCall(
                    position,
                    customDialogThreeButtons.getValue(),
                    getString(R.string.Faltas)
                )

            }.setOkButtonTwo(getString(R.string.BasuraRechazar)) {

                checkAndCall(
                    position,
                    customDialogThreeButtons.getValue(),
                    getString(R.string.BasuraRechazar)
                )

            }.setOkButtonThree(getString(R.string.Reject)) {
                checkAndCall(
                    position,
                    customDialogThreeButtons.getValue(),
                    getString(R.string.Reject)
                )

            }.setOkButtonFour(getString(R.string.Split)) {
                checkAndCall(
                    position,
                    customDialogThreeButtons.getValue(),
                    getString(R.string.Split)
                )


            }.setOkButtonAdd(getString(R.string.Agregar)) {
                checkAndCall(
                    position,
                    customDialogThreeButtons.getValue(),
                    getString(R.string.Agregar)
                )

            }.setKoButton("Cancelar") {
                scanRequest()
                customDialogThreeButtons.dismiss()
            }.show()
    }

    private fun checkAndCall(position: Int, value: String, action: String) {
        if (value.trim().isNullOrEmpty()) {
            getString(R.string.Indicanuevacantidad).toast(requireContext())
        } else {

            when (action) {
                getString(R.string.Agregar) -> increaseQuantity(position, value)
                getString(R.string.BasuraRechazar) -> TrashMissingReject(position, value,"TRUE")
                getString(R.string.Faltas) -> TrashMissingReject(position, value,"FALSE")
                getString(R.string.Reject) -> TrashMissingReject(position, value,"reject")
                getString(R.string.Split) -> split(position, value)

            }

            scanRequest()
            customDialogThreeButtons.dismiss()
        }
    }

    private fun split(position: Int, quantity: String) {

        quantityCollectionSplit=quantity
        positionCollectionSplit=position

        var totalQuantity: Int = 0
        try {
            totalQuantity = sales[position].quantity.toInt() - quantity.toInt()
        } catch (e: Exception) {
        }
        viewModel.saleMove(
            getData(USER),
            getData(PASSWORD),
            saleFk = sales[position].saleFk,
            quantity = totalQuantity.toString(),
            originalQuantity = sales[position].quantity,
            "saleMove"
        )

       /* sales[position].quantity = quantity

        if (quantity == "0")
            markLine(position, type)
        saleAdapter!!.notifyDataSetChanged()

        //enviar mensaje a salix
        val ticket =
            "[" + sales[position].ticketFk + "](https://salix.verdnatura.es/#!/ticket/" + sales[position].ticketFk + "/summary)"
        val message =
            "Se ha enviado a Split el articulo " + sales[position].itemFk + " del ticket " + ticket
        viewModel.sendChekingPresence(
            token = getData(TOKEN),
            workerId = sales[position].salePersonFk,
            message = message, "sendChekingPresence"
        )*/

    }

    private fun TrashMissingReject(
        position: Int,
        quantity: String,
        typeCollectionTrashMissingReject: String
    ) {
        typeCollectionMissing = typeCollectionTrashMissingReject
        positionCollectionMissing = position
        quantityCollectionMissing = quantity

        var totalQuantity: Int = 0
        try {
            totalQuantity = sales[position].quantity.toInt() - quantity.toInt()
        } catch (e: Exception) {
        }

        viewModel.collectionMissingTrash(
            getData(USER),
            getData(PASSWORD),
            saleFk = sales[position].saleFk,
            quantity = totalQuantity.toString(),
            warehouseFk = getData(WAREHOUSEFK),
            type = typeCollectionTrashMissingReject,
            originalQuantity = quantity,
            callFunction = "collectionMissingTrash"
        )
     /*   sales[position].quantity = quantity
        saleAdapter!!.notifyDataSetChanged()
        if (quantity == "0")
            markLine(position, type)

*/
    }

   /* private fun trash(position: Int, quantity: String) {
        var totalQuantity: Int = 0
        try {
            totalQuantity = sales[position].quantity.toInt() - quantity.toInt()
        } catch (e: Exception) {
        }
        viewModel.collectionMissingTrash(
            usuario = user,
            password = password,
            saleFk = sales[position].saleFk,
            quantity = totalQuantity.toString(),
            warehouseFk = warehouseFk,
            type = "FALSE",
            originalQuantity = quantity,
            callFunction = "collectionMissingTrash"
        )

        sales[position].quantity = quantity
        saleAdapter!!.notifyDataSetChanged()
        if (quantity == "0")
            markLine(position, type)


        //enviar mensaje a salix
        val ticket =
            "[" + sales[position].ticketFk + "](https://salix.verdnatura.es/#!/ticket/" + sales[position].ticketFk + "/summary)"
        val message =
            "Se ha enviado a Faltas la cantidad de " + totalQuantity + " del articulo " + sales[position].itemFk + "  ticket " + ticket
        viewModel.sendChekingPresence(
            token = token,
            workerId = sales[position].salePersonFk,
            message = message, "sendChekingPresence"
        )
    }

    private fun missing(position: Int, quantity: String) {
        var totalQuantity: Int = 0
        try {
            totalQuantity = sales[position].quantity.toInt() - quantity.toInt()
        } catch (e: Exception) {
        }
        viewModel.collectionMissingTrash(
            usuario = user,
            password = password,
            saleFk = sales[position].saleFk,
            quantity = totalQuantity.toString(),
            warehouseFk = warehouseFk,
            type = "TRUE",
            originalQuantity = quantity,
            callFunction = "collectionMissingTrash",
        )
        //sales[position].originalQuantity = quantity
        sales[position].quantity = quantity
        //sales[position].startQuantity = quantity

        if (quantity == "0")
            markLine(position, type)
        saleAdapter!!.notifyDataSetChanged()

        //enviar mensaje a salix
        val ticket =
            "[" + sales[position].ticketFk + "](https://salix.verdnatura.es/#!/ticket/" + sales[position].ticketFk + "/summary)"
        val message =
            "Se ha enviado a Basura " + totalQuantity + " del articulo " + sales[position].itemFk + "  ticket " + ticket
        viewModel.sendChekingPresence(
            token = token,
            workerId = sales[position].salePersonFk,
            message = message,
            "sendChekingPresence"
        )
    }

    private fun reject(position: Int, quantity: String) {
        positionReject = position
        quantityReject = quantity

        var totalQuantity: Int = 0
        try {
            totalQuantity = sales[position].quantity.toInt() - quantity.toInt()
        } catch (e: Exception) {
        }
        viewModel.collectionMissingTrash(
            usuario = user,
            password = password,
            saleFk = sales[position].saleFk,
            quantity = totalQuantity.toString(),
            warehouseFk = warehouseFk,
            type = "reject",
            originalQuantity = quantity, callFunction = "collectionMissingTrash"
        )

        //sales[position].originalQuantity = quantity
        sales[position].quantity = quantity
        //sales[position].startQuantity = quantity


        saleAdapter!!.notifyDataSetChanged()
        if (quantity == "0")
            markLine(position, type)

        //enviar mensaje a salix
        /* val ticket =
             "[" + sales[position].ticketFk + "](https://salix.verdnatura.es/#!/ticket/" + sales[position].ticketFk + "/summary)"
         val message =
             "Se ha modificado la cantidad original " + sales[position].originalQuantity + " del artículo " + sales[position].itemFk + " a nueva cantidad: " + sales[position].quantity + " del ticket " + ticket
         viewModel.sendChekingPresence(
             token = token,
             workerId = sales[position].salePersonFk,
             message = message, "sendChekingPresence"
         )*/
    }*/

    private fun missingTrashSuccesful() {

        var message=""

        sales[positionCollectionMissing].quantity = quantityCollectionMissing
        saleAdapter!!.notifyDataSetChanged()
        if (quantityCollectionMissing == "0")
            markLine(positionCollectionMissing, type)

        var totalQuantity: Int = 0
        try {
            totalQuantity = sales[positionCollectionMissing].originalQuantity.toInt() - quantityCollectionMissing.toInt()
        } catch (e: Exception) {
        }

            saleAdapter!!.notifyDataSetChanged()

            val ticket =
                "[" + sales[positionCollectionMissing].ticketFk + "](https://salix.verdnatura.es/#!/ticket/" + sales[positionCollectionMissing].ticketFk + "/summary)"


        when (typeCollectionMissing){

            "TRUE"->{
                message = "Se ha enviado a Basura " + totalQuantity + " del articulo " + sales[positionCollectionMissing].itemFk + "  ticket " + ticket
            }
            "FALSE"->{
                message= "Se ha enviado a Faltas la cantidad de " + totalQuantity + " del articulo " + sales[positionCollectionMissing].itemFk + "  ticket " + ticket
            }
            "reject"->{
                message = "Se ha modificado la cantidad de  " + sales[positionCollectionMissing].originalQuantity + " del artículo " + sales[positionCollectionMissing].itemFk + " a nueva cantidad: " + sales[positionCollectionMissing].quantity + " del ticket " + ticket

            }
        }

            sendSalixMessageNew(message, sales[positionCollectionMissing].salePersonFk)

        binding.splashProgress.visibility = VISIBLE
        viewModel.collectionTicketGet(
            getData(USER),
            getData(PASSWORD),
            collection.collectionFk,
            getData(SECTORFK),
            type,
            "collection_getTickets"
        )

    }

    private fun incresaseSuccesful() {
        try {
            //val quantityPicked = (quantity.toInt() - sales[position].quantity.toInt()).toString()
            //sales[position].originalQuantity = quantity
            sales[positionIncreaseQuantity].quantity = quantityIncrease
            /*if (type == CONTROLADOR){
                   showShelving(position,0,quantityPicked)
               }*/
        } catch (e: Exception) {
        }
        saleAdapter?.notifyDataSetChanged()
        //enviar mensaje a salix

        val ticket =
            "[" + sales[positionIncreaseQuantity].ticketFk + "](https://salix.verdnatura.es/#!/ticket/" + sales[positionIncreaseQuantity].ticketFk + "/summary)"
        val message =
            "Se ha modificado la cantidad original " + sales[positionIncreaseQuantity].originalQuantity + " del artículo " + sales[positionIncreaseQuantity].itemFk + " a nueva cantidad: " + sales[positionIncreaseQuantity].quantity + " del ticket " + ticket

        sendSalixMessageNew(message, sales[positionIncreaseQuantity].salePersonFk)
        /* val ticket =
             "[" + sales[positionIncreaseQuantity].ticketFk + "](https://salix.verdnatura.es/#!/ticket/" + sales[positionIncreaseQuantity].ticketFk + "/summary)"
         val message =
             "Se ha modificado la cantidad original " + sales[positionIncreaseQuantity].originalQuantity + " del artículo " + sales[positionIncreaseQuantity].itemFk + " a nueva cantidad: " + sales[positionIncreaseQuantity].quantity + " del ticket " + ticket
          viewModel.sendChekingPresence(
             token = token,
             workerId = sales[positionIncreaseQuantity].salePersonFk,
             message = message, "sendChekingPresence"
         )*/


    }


    private fun sendSalixMessageNew(message: String, workerId: String) {

        viewModel.sendChekingPresence(
            token = getData(TOKEN),
            workerId = workerId,
            message = message,
            "sendChekingPresence"
        )
    }

    private fun increaseQuantity(position: Int, quantity: String) {
        positionIncreaseQuantity = position
        quantityIncrease = quantity

        viewModel.collectionIncreaseQuantity(
            getData(USER),
            getData(PASSWORD),
            saleFk = sales[position].saleFk,
            quantity = quantity, "collection_increaseQuantity"
        )


    }

    //ESTADOS
    private fun changeTicketState() {
         if (type == CONTROLADOR) {
             tickets.forEach {
                 viewModel.ticket_setState(
                     usuario=getData(USER),
                     password = getData(PASSWORD),
                     ticketFk = it,
                     state = "CHECKED",
                     callFunction = "ticket_setState"
                 )
             }
         } else if (type == SACADOR) {
             tickets.forEach {
                 viewModel.ticket_setState(
                     usuario=getData(USER),
                     password = getData(PASSWORD),
                     ticketFk = it,
                     state = "PREPARED",
                     callFunction = "ticket_setState"
                 )
             }

         }
     }

  /*  private fun changeInitTicketState() {
        if (type == CONTROLADOR) {
            tickets.forEach {
                viewModel.collectionUpdateState(
                    getData(USER),
                    getData(PASSWORD),
                    collectionFk = it,
                    state = ON_CHECKING, "collection_updateState"
                )
            }
        } else if (type == SACADOR) {
            tickets.forEach {
                viewModel.collectionUpdateState(
                    getData(USER),
                    getData(PASSWORD),
                    collectionFk = it,
                    state = ON_PREPARATION, "collection_updateState"
                )
            }
        }
    }*/

    //OTROS
    private fun showScanner(index: Int, sale: SaleVO) {

        Log.d("VERDNATURA", "Entrant en el show scanner al punxar sobre la sale")
        customDialogInput.setTitle("" + sale.itemFk)
            .setDescription(getString(R.string.Escaneaelcarroparaelitemseleccionado))
            .setOkButton("Aceptar") {
                if (!customDialogInput.getValue().isNullOrEmpty()) {
                    findSale(customDialogInput.getValue(), index)
                }
                customDialogInput.setValue("")
                scanRequest()
                customDialogInput.dismiss()
                hideKeyboards()
            }.setKoButton("Cancelar") {
                customDialogInput.dismiss()
            }.setValue("").show()
        customDialogInput.getEditText().requestFocus()
        customDialogInput.getEditText().setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                if (!customDialogInput.getValue().isNullOrEmpty()) {
                    findSale(customDialogInput.getValue(), index)
                }
                customDialogInput.setValue("")
                scanRequest()
                customDialogInput.dismiss()
                hideKeyboards()
                return@setOnEditorActionListener true
            }
            false
        }

    }

    private fun showErrorMessage(text: String) {
        customDialog.setTitle("Error al marcar la linea").setDescription(text)
            .setKoButton("Cerrar") {
                customDialog.dismiss()
            }.show()
    }

    private fun showMistakeList(list: List<MistakeTypeVO>) {
        if (binding.splashProgress != null) binding.splashProgress.visibility = View.GONE
        listPlacementSupply = ArrayList()
        list.forEach {
            listPlacementSupply.add(BarcodeVO(code = it.description))
        }


        customDialogList.setTitle("Causa de error").setKoButton("Cancelar") {
            scanRequest()
            hideKeyboards()
            customDialogList.dismiss()
        }.hideDialog().show()


        placementSupplyAdapter =
            BarcodeAdapter(listPlacementSupply, object : OnBarcodeRowClickListener {
                override fun onBarcodeRowClickListener(item: BarcodeVO) {
                    list.forEach {
                        if (it.description == item.code) {
                            viewModel.saleMistakeAdd(usuario = getData(USER),
                        password = getData(PASSWORD),
                                vSaleFk = mistakeSale?.saleFk!!,
                                vUserFk = mistakeSale?.workerFk!!,
                                vTypeFk = it.id, "saleMistake_Add"
                            )
                            "Causa del error registrado".toast(requireContext())
                            customDialogList.dismiss()
                        }
                    }

                }
            }, showDelete = false)

        customDialogList.getRecyclerView().adapter = placementSupplyAdapter

        customDialogList.getRecyclerView().layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)
    }

    fun isNumber(num: String): Boolean {
        var numberInt = 0
        try {
            numberInt = num.toInt()
            return true
        } catch (e: Exception) {
            return false
        }
    }

    //VOZ
    override fun onResults(results: Bundle) {
        super.onResults(results)
        Log.i("Speech", "onResults")
        val matches = results
            .getStringArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
        if (matches != null) {
            Log.i("RESPUESTA", matches[0])
            mIsListening = false
            mSpeechRecognizer!!.cancel()
            getText(matches[0])
        }
    }

    private fun getText(text: String) {
        val pos = checkText(text)
        when (pos) {
            NEW_COLLECTION -> {

            }
            CANCEL -> {

            }
            LISTO -> {
                state += 1
                speakOrder()
            }
            VOLVER -> {

            }
            REPITE -> {
                speakOrder()
            }
            OTRO -> {
                speak("Orden no registrada")
            }
            FALTA -> {
                speak("Orden no registrada")
            }
            else -> speak("Orden no registrada")
        }
    }

    private fun speakOrder() {
        if (type == SACADOR) {
            if (state == 0) {
                speak("Colección cargada.")
            } else {
                var index = 0
                for (sale in sales) {
                    if (sale.isPrepared == "0" && sale.isPreviousPrepared == "0" && sale.isControlled == "0") {
                        if (state == 1) {
                            //cantar pasillo
                            if (sale.placements.size > 0) {
                                var placementIndex = 0
                                for (placement in sale.placements) {
                                    var visible = placement.visible
                                    visible = visible.replace("(", "")
                                    visible = visible.replace(")", "")
                                    var quantityVisible = 0
                                    try {
                                        quantityVisible = visible.toInt()
                                    } catch (e: Exception) {
                                    }

                                    if (quantityVisible > 0) {
                                        val pasillo = placement.placement.subSequence(
                                            0,
                                            sale.placements[0].placement.indexOf("-")
                                        ).toString()
                                        val sector = placement.placement.subSequence(
                                            sale.placements[0].placement.indexOf("-") + 1,
                                            sale.placements[0].placement.length
                                        ).toString()
                                        var letras: ArrayList<Char> = ArrayList()
                                        val carro = placement.shelving
                                        var carroSeparate: StringBuilder = StringBuilder("")
                                        for (l in carro) {
                                            carroSeparate.append(l)
                                            carroSeparate.append(" ")
                                        }


                                        placementPicked = placement
                                        storedShelvingPosition = placementIndex

                                        if (lastPlacementPicked == null || lastPlacementPicked!!.placement != placement.placement) {
                                            lastPlacementPicked = placement
                                            try {
                                                speak("Pasillo " + pasillo.toInt() + ", Sector " + sector.toInt() + ",  " + carroSeparate.toString())
                                            } catch (e: Exception) {
                                                speak("Pasillo " + pasillo + " Sector " + sector + " Carro " + carro)
                                            }
                                        } else if (lastPlacementPicked != null && lastPlacementPicked!!.shelving != placement.shelving) {
                                            speak(" Carro " + carro)
                                        } else {
                                            state = 2
                                            speakOrder()
                                        }

                                        break
                                    }

                                    placementIndex += 1
                                }

                            } else {
                                speak("El item " + sale.itemFk + " no se encuentra disponible. Se necesita actuación manual.")
                            }

                        } else if (state == 2) {
                            //cantar articulo
                            try {
                                val item = sale.itemFk
                                val cantidad = sale.quantity.toInt() - sale.pickedQuantity.toInt()
                                val listtext = sale.longName.split(" ")
                                var nombre: String
                                if (listtext.isNotEmpty()) {
                                    nombre = listtext.get(0) + " " + sale.color
                                } else {
                                    nombre = sale.longName + " " + sale.color
                                }

                                speak("Item " + item.toInt() + " " + nombre + " , Cantidad: " + cantidad)
                            } catch (e: Exception) {
                                speak("Se necesita actuación manual")
                            }
                        } else if (state == 3) {
                            //marcar linea
                            var visible = placementPicked!!.visible
                            visible = visible.replace("(", "")
                            visible = visible.replace(")", "")
                            var quantityVisible = 0
                            try {
                                quantityVisible = visible.toInt()
                            } catch (e: Exception) {
                            }

                            var needPicked = 0
                            try {
                                needPicked = sale.quantity.toInt() - sale.pickedQuantity.toInt()
                            } catch (e: Exception) {
                            }

                            storedPosition = index
                            if (quantityVisible < needPicked) {
                                onQuantityOfShelvingSelected(
                                    itemShelvingFk = placementPicked!!.shelving,
                                    quantity = quantityVisible.toString()
                                )
                            } else {
                                onQuantityOfShelvingSelected(
                                    itemShelvingFk = placementPicked!!.shelving,
                                    quantity = needPicked.toString()
                                )
                            }

                            state = 0
                            //speak("Acción registrada. Diga listo para continuar")
                            speak("Registrada")

                        }

                        break

                    }

                    index += 1
                }

                if (index >= sales.size) {
                    speak("Colección completada.")
                }
            }
        }
    }

    override fun onDestroy() {
        cancelSpeech()
        super.onDestroy()
    }

}
