package es.verdnatura.presentation.view.feature.controlvehiculo.fragment

import android.content.Context
import android.graphics.drawable.Drawable
import android.os.Build
import android.util.TypedValue
import android.view.View.*
import android.view.inputmethod.EditorInfo
import android.widget.ImageView
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentVehiclecontrolBinding
import es.verdnatura.domain.toast
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.*
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.component.CustomDialogInput
import es.verdnatura.presentation.view.component.CustomDialogThreeButtons
import es.verdnatura.presentation.view.feature.historicovehiculo.fragment.HistoricoVehiculoFragment
import es.verdnatura.presentation.view.feature.pasillero.model.PasillerosItemVO
import es.verdnatura.presentation.view.feature.sacador.fragment.SacadorFragment
import java.text.SimpleDateFormat
import java.util.*
import kotlin.collections.ArrayList

class ControlVehiculoFragment(
    var tagName: String = ""
) : BaseFragment<FragmentVehiclecontrolBinding, ControlVehiculoViewModel>(
    ControlVehiculoViewModel::class
) {

    private var goBack: Boolean = false
    private var onCollectionSelectedListener: OnCollectionSelectedListener? = null
    override fun getLayoutId(): Int = R.layout.fragment_vehiclecontrol
    private lateinit var customDialog: CustomDialog
    private var isWorkedTimed = "UNREGISTERED"
    private lateinit var customDialogInput: CustomDialogInput
   private lateinit var customDialogThreeButtons: CustomDialogThreeButtons

    companion object {
        fun newInstance(tagName: String) = ControlVehiculoFragment(tagName)
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        if (context is OnCollectionSelectedListener) onCollectionSelectedListener = context
    }

    override fun init() {

        customDialog = CustomDialog(requireContext())
        customDialogInput = CustomDialogInput(requireContext())
        customDialogThreeButtons = CustomDialogThreeButtons(requireContext())
        goBack = false

        binding.splashProgress.visibility = GONE

        /*  if (getWagons() < 2 && !tagName.equals(SacadorFragment.TAG))
          {
                  ma.openFragmentPickers()
          }*/


        if (tagName.equals(HistoricoVehiculoFragment.TAG)) {
            setToolBar("HISTORY")
            showScanner("Escanea matrícula para ver histórico", "HISTORY")
        } else {

            checkControlTimeVehicle()
            setToolBar("ALL")
        }


        setEvents()
        super.init()
    }



    override fun onPause() {
        goBack = true
        super.onPause()
    }

    private fun setToolBar(direction: String?) {

        binding.mainToolbar.toolbarTitle.text = getString(R.string.vehiclecontrol)
        binding.mainToolbar.toolbarTitle.setTextSize(TypedValue.COMPLEX_UNIT_SP, 13F)

        val listIcons: ArrayList<ImageView> = ArrayList()
        val iconHistory = ImageView(context)
        iconHistory.setImageResource(R.drawable.ic_history_black_24dp)
        val iconVehicleIn = ImageView(context)
        iconVehicleIn.setImageResource(R.drawable.car)
        val iconVehicleOut = ImageView(context)
        iconVehicleOut.setImageResource(R.drawable.car_off)


        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {

            iconHistory.tooltipText=getTooltip(R.drawable.ic_history_black_24dp)
            iconVehicleIn.tooltipText=getTooltip(R.drawable.car)
            iconVehicleOut.tooltipText=getTooltip(R.drawable.car_off)

        }

    when (direction){
        "OUT"-> {
            listIcons.remove(iconVehicleIn)
            listIcons.add(iconVehicleOut)
            listIcons.add(iconHistory)
        }
        "IN"->{
                listIcons.remove(iconVehicleOut)
                listIcons.add(iconVehicleIn)
                listIcons.add(iconHistory)
            }
    }


        binding.mainToolbar.toolbarIcons.adapter = ToolBarAdapterTooltip(listIcons, object : OnOptionsSelectedListener {
            override fun onOptionsItemSelected(item: Drawable) {

                when (item){
                    iconHistory.drawable->  showScanner("Escanea matrícula para ver histórico", "HISTORY")
                    iconVehicleIn.drawable->  showScanner("Escanea matrícula del vehículo", "IN")
                    iconVehicleOut.drawable-> showScanner("Escanea matrícula para dejar vehículo", "OUT")
                }
            }
        })
        binding.mainToolbar.toolbarIcons.layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.HORIZONTAL, false)
    }

    private fun showScanner(description: String, action: String) {

        customDialogInput.setTitle(getString(R.string.vehiclecontrol)).setDescription(description)
            .setOkButton("Aceptar") {
                if (customDialogInput.getValue().isNotEmpty()) {
                    // "$description ${customDialogInput.getValue()} OK".toast(requireContext())
                    insertControlTimeVehicle(customDialogInput.getValue(), action)
                }
                customDialogInput.setValue("")
                scanRequest()
                customDialogInput.dismiss()
                hideKeyboards()
            }

            .setKoButton("Cancelar") {
                customDialogInput.dismiss()
                if (tagName.equals(HistoricoVehiculoFragment.TAG)) {
                ma.onBackPressed()
                }

            }.setValue("").show()

        customDialogInput.setFocusText()

        customDialogInput.getEditText().setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {

                if (customDialogInput.getValue().isNotEmpty()) {

                    insertControlTimeVehicle(customDialogInput.getValue(), action)
                }
                customDialogInput.setValue("")
                scanRequest()
                customDialogInput.dismiss()
                hideKeyboards()
                return@setOnEditorActionListener true
            }
            false
        }

    }

    private fun checkControlTimeVehicle() {

        if (!goBack) {
        //  d("la fecha es "+getData("DATENOVEHICLE"))
           //if (!IsNoVehicleDataToday()){
            binding.splashProgress.visibility = VISIBLE
            viewModel.workerMachinery_isRegistered(getData(USER),getData(PASSWORD),"workerMachinery_isRegistered")
        /*}else{
            ma.openFragmentPickers()
        }*/
        }
    }

    private fun insertControlTimeVehicle(plateNumber: String, direction: String) {

        if (direction.equals("HISTORY")) {
            ma.onPasillerosItemClickListener(
                PasillerosItemVO(title = "Historico Vehículo"),
                plateNumber
            )
        } else if (direction.equals("IN")) {
            binding.splashProgress.visibility = VISIBLE
            viewModel.machineWorker_add(getData(USER),getData(PASSWORD), plateNumber,"machineWorker_add")

        } else if (direction.equals("OUT")) {
            viewModel.machineWorker_update(getData(USER),getData(PASSWORD), plateNumber,"machineWorker_update")

        } else if (direction.equals("")) {
            ma.openFragmentPickers(true)
        }

    }

    private fun scanRequest() {
      //  if (binding.scanInput != null) {
            binding.scanInput.requestFocus()
        //}
        hideKeyboards()
    }

    private fun hideKeyboards() {
        try {
            requireActivity().hideKeyboard()
        } catch (e: Exception) {
        }
    }


    private fun setEvents() {

        binding.mainToolbar.backButton.setOnClickListener {
            ma.openFragmentPickers(true)
            //requireActivity().onBackPressed()

        }

        binding.scanInput.requestFocus()
        binding.scanInput.setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                if (!binding.scanInput.text.isNullOrEmpty()) {

                    if (tagName.equals(HistoricoVehiculoFragment.TAG)) {
                        showOptionsHistorical(binding.scanInput.text.toString())
                    } else {
                        showOptionsVehicleControl(binding.scanInput.text.toString())
                    }
                }
                binding.scanInput.setText("")
                ma.hideKeyboard(binding.scanInput)
                return@setOnEditorActionListener true
            }
            false
        }
    }

    private fun showOptionsVehicleControl(plate: String) {

        customDialogThreeButtons.setCustomDialogValue(GONE)

        customDialogThreeButtons.setDescription("¿Qué desea hacer con el vehículo de matrícula $plate")
            .setValue(plate)
        if (isWorkedTimed.equals("UNREGISTERED")) {
            customDialogThreeButtons.setOkButtonTwo("COGER") {
                if (customDialogThreeButtons.getValue().trim().isEmpty()) {
                    "Vuelva a escanear".toast(requireContext())
                } else {
                    binding.splashProgress.visibility = VISIBLE
                    customDialogThreeButtons.dismiss()
                    insertControlTimeVehicle(customDialogThreeButtons.getValue(), "IN")
                }
            }
        } else {

            customDialogThreeButtons.setOkButtonThree("DEJAR") {
                if (customDialogThreeButtons.getValue().trim().isEmpty()) {
                    "Vuelva a escanear".toast(requireContext())
                } else {
                    binding.splashProgress.visibility = VISIBLE
                    customDialogThreeButtons.dismiss()
                    insertControlTimeVehicle(customDialogThreeButtons.getValue(), "OUT")
                }
            }
        }.setOkButtonFour("HISTÓRICO") {
            if (customDialogThreeButtons.getValue().trim().isEmpty()) {
                "Vuelva a escanear".toast(requireContext())
            } else {
                customDialogThreeButtons.dismiss()
                insertControlTimeVehicle(customDialogThreeButtons.getValue(), "HISTORY")

            }

        }.setKoButton("Cancelar") {
            scanRequest()
            customDialogThreeButtons.dismiss()
        }.show()
    }

    private fun showOptionsHistorical(plate: String) {

        customDialogThreeButtons.setCustomDialogValue(GONE)

        customDialogThreeButtons.setDescription("¿Qué desea hacer con el vehículo de matrícula $plate")
            .setValue(plate)

            .setOkButtonFour("HISTÓRICO") {
                if (customDialogThreeButtons.getValue().trim().isEmpty()) {
                    "Vuelva a escanear".toast(requireContext())
                } else {
                    customDialogThreeButtons.dismiss()
                    insertControlTimeVehicle(customDialogThreeButtons.getValue(), "HISTORY")

                }

            }.setKoButton("Cancelar") {
                scanRequest()
                customDialogThreeButtons.dismiss()
            }.show()
    }

    override fun observeViewModel() {

        with(viewModel) {
            binding.splashProgress.visibility = GONE
            responseinsert.observe(viewLifecycleOwner,  {

                if (it.isError) {
                    ma.messageWithSound(it.errorMessage,true,false)
                    /*customDialog.setTitle("Error").setDescription(it.errorMessage)
                        .setOkButton("Cerrar") {
                            customDialog.dismiss()
                        }.show()*/
                } else {
                    if (it.response == "false") {

                        customDialog.setTitle("Error")
                            .setDescription("Revisar llamada del registro de vehículos")
                            .setOkButton("Cerrar") {
                                customDialog.dismiss()
                            }.show()

                    } else {
                        if (it.response == "true") {
                            //Log.i(TD,"insertado registro")
                            //"Acción registrada correctamente".toast(context)
                            /*   customDialog.setTitle("Llamada correcta")
                                   .setDescription("Acción registrada satisfactoriamente")
                                   .setOkButton("Cerrar") {
                                       customDialog.dismiss()
                                   }.show()*/
                            ma.openFragmentPickers(true)
                            //ma.onPasillerosItemClickListener( PasillerosItemVO(title = "Sacadores"),"CONTROL")
                            //checkControlTimeVehicle()
                        }

                    }
                }
                binding.splashProgress.visibility = GONE
            })

            responsecontrol.observe(viewLifecycleOwner,  {
                binding.splashProgress.visibility = GONE
                // Log.i("VERDNATURA:","${it.response}")
                if (it.isError) {
                    ma.messageWithSound(it.errorMessage,true,false)
                    /*customDialog.setTitle("Error").setDescription(it.errorMessage)
                        .setOkButton("Cerrar") {
                            customDialog.dismiss()
                        }.show()*/
                } else {
                    if (it.response.equals("1")) {
                        isWorkedTimed = "REGISTERED"
                        setToolBar("OUT")
                        /*  customDialog.setTitle("Control de vehículos")
                              .setDescription("SI Tienes coche asignado")
                              .setOkButton("Cerrar") {
                                  customDialog.dismiss()
                              }.show()*/
                        if (!tagName.equals(SacadorFragment.TAG)) {
                            ma.openFragmentPickers(true)
                        }

                    } else {

                        if (it.response.equals("0")) {
                            setToolBar("IN")
                           /* customDialog.setTitle("Control de vehículos")
                                .setDescription("Escanea la matrícula del vehículo o elige sin vehículo")
                                .setOkButton("Escanear vehículo") {
                                    customDialog.dismiss()
                                    showScanner("Escanea matrícula del vehículo", "IN")
                                }
                                .setKoButton("SIN VEHÍCULO") {
                                    customDialog.dismiss()
                                    saveData(DATENOVEHICLE,   SimpleDateFormat("yyyy-MM-dd",Locale.getDefault()).format(Date()).toString())
                                    ma.openFragmentPickers(true)

                                }.show()*/
                            customDialogInput.setTitle(getString(R.string.vehiclecontrol)).setDescription("Escanea la matrícula del vehículo o elige sin vehículo")
                                .setOkButton("Coger vehículo") {
                                    if (customDialogInput.getValue().isNotEmpty()) {
                                        // "$description ${customDialogInput.getValue()} OK".toast(requireContext())
                                        insertControlTimeVehicle(customDialogInput.getValue(), "IN")
                                    }
                                    customDialogInput.setValue("")
                                    scanRequest()
                                    customDialogInput.dismiss()
                                    hideKeyboards()
                                }

                                .setKoButton("Sin vehículo") {
                                    customDialogInput.dismiss()
                                    ma.openFragmentPickers(true)
                                   /* if (tagName.equals(HistoricoVehiculoFragment.TAG)) {
                                        ma.onBackPressed()
                                    }*/

                                }.setValue("").show()

                                customDialogInput.setFocusText()

                        }
                    }
                }
            })

        }

    }

    private fun IsNoVehicleDataToday(): Boolean {

        return (getData(DATENOVEHICLE)==SimpleDateFormat("yyyy-MM-dd",Locale.getDefault()).format(Date()))
    }


}


