package es.verdnatura.presentation.view.feature.paletizador.fragment


import android.content.Context
import android.graphics.drawable.Drawable
import android.view.View
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentExpeditionPalletDetailBinding
import es.verdnatura.domain.notNull
import es.verdnatura.domain.toast
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.OnOptionsSelectedListener
import es.verdnatura.presentation.common.OnPalletClickListener
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.component.CustomDialogList
import es.verdnatura.presentation.view.feature.inventario.adapter.ToolBarAdapter
import es.verdnatura.presentation.view.feature.main.activity.MainActivity
import es.verdnatura.presentation.view.feature.paletizador.adapter.ExpeditionPalletViewAdapter
import es.verdnatura.presentation.view.feature.paletizador.model.ItemExpeditionTruckVO
import es.verdnatura.presentation.view.feature.paletizador.model.ItemPalletVO
import es.verdnatura.presentation.view.feature.paletizador.model.ItemPalletViewListVO
import java.text.SimpleDateFormat
import java.util.*
import kotlin.collections.ArrayList

class ExpeditionPalletDetailFragment(
    var itemExpeditionTruckVO: ItemExpeditionTruckVO? = null,
    var itemPallet: ItemPalletVO? = null
) : BaseFragment<FragmentExpeditionPalletDetailBinding, ExpeditionPalletDetailViewModel>(
    ExpeditionPalletDetailViewModel::class) {

    private var adapter : ExpeditionPalletViewAdapter? = null
   // private lateinit var customDialogList: CustomDialogList
    private var onPalletClickListener:OnPalletClickListener? = null

    private lateinit var customDialog: CustomDialog


    companion object {
        fun newInstance(item: ItemExpeditionTruckVO,itemPallet: ItemPalletVO) =
            ExpeditionPalletDetailFragment(item,itemPallet)
    }

    override fun onAttach(context: Context) {
        if (context is OnPalletClickListener) onPalletClickListener = context
        super.onAttach(context)
    }

    override fun getLayoutId(): Int = R.layout.fragment_expedition_pallet_detail

    override fun init() {
        customDialog = CustomDialog(requireContext())
        //customDialogList = CustomDialogList(requireContext())
        binding.expeditionPalletDetailPallet.text = "Pallet "+itemPallet!!.Pallet
        binding.splashProgress.visibility = View.VISIBLE
        //requireActivity().main_bottom_navigation.visibility = View.GONE
         ma.hideBottomNavigation(View.GONE)
        binding.mainToolbar.toolbarTitle.text = getCURDATE(itemExpeditionTruckVO!!.ETD) + " " + itemExpeditionTruckVO!!.Destino

        setToolBar()
        setEvents()
        viewModel.expeditionPallet_List(getData(USER),
            getData(PASSWORD),itemPallet!!.Pallet,"expeditionPallet_View")
        super.init()
    }

    private fun setEvents(){
        binding.mainToolbar.backButton.setOnClickListener {
            requireActivity().onBackPressed()
        }

        binding.buttonOk.setOnClickListener {
            requireActivity().onBackPressed()
            // sergio: en caso de que hay problemas habrá que cambiar aquí la llamada
            // MainActivity.addFragmentOnTop(ExpeditionPalletDetailFragment.newInstance(itemTruck,itemPallet))
            // que llame diractamente al fragment para que no se repita la ultima accion
            ////Log.i("VERDNATURA:", "pulsamos boton OK en detail fragment")

        }
    }

    private fun setToolBar(){
        val listIcons:ArrayList<Drawable> = ArrayList()
        val iconReload : Drawable = resources.getDrawable(R.drawable.ic_autorenew_black_24dp,resources.newTheme())
        val iconEditar : Drawable = resources.getDrawable(R.drawable.ic_mode_edit_black_24dp,resources.newTheme())
        val iconBorrar : Drawable = resources.getDrawable(R.drawable.ic_delete_forever_black_24dp,resources.newTheme())
        val iconPrint : Drawable = resources.getDrawable(R.drawable.ic_print_black_24dp,resources.newTheme())
        listIcons.add(iconEditar)
        listIcons.add(iconBorrar)
        listIcons.add(iconPrint)
        binding.mainToolbar.toolbarIcons.adapter = ToolBarAdapter(listIcons,object: OnOptionsSelectedListener {
            override fun onOptionsItemSelected(item: Drawable) {
                if (item == iconReload){
                    binding.splashProgress.visibility = View.VISIBLE
                    viewModel.expeditionPallet_List(getData(USER),
                        getData(PASSWORD),itemPallet!!.Pallet,"expeditionPallet_View")
                }else if(item == iconEditar){
                    onPalletClickListener!!.onPalletClickListener(itemExpeditionTruckVO!!,itemPallet!!)
                }else if(item == iconBorrar){
                    binding.splashProgress.visibility = View.VISIBLE
                    viewModel.expeditionPalletDel(getData(USER),
                        getData(PASSWORD),itemPallet!!.Pallet,"expeditionPallet_Del")
                }else if(item == iconPrint){
                    binding.splashProgress.visibility = View.VISIBLE

                    (getString(R.string.Imprimiendo)+" por " +getData(SECTORDESCRIP)).toast(requireContext())
                    viewModel.expeditionPalletPrintSet(getData(USER),
                        getData(PASSWORD),itemPallet!!.Pallet,getData(SECTORFK),"expeditionPallet_PrintSet")

                }
            }
        })
        binding.mainToolbar.toolbarIcons.layoutManager = LinearLayoutManager(requireContext(), LinearLayoutManager.HORIZONTAL, false)
    }

    private fun getCURDATE(date:String):String{
        val c = Calendar.getInstance()
        val df = SimpleDateFormat("yyyy-mm-dd HH:mm")
        val df2 = SimpleDateFormat("HH:mm")
        c.time = df.parse(date)
        return df2.format(c.time)
    }

    override fun observeViewModel() {
        with(viewModel){
            loadExpeditionPalletList.observe(viewLifecycleOwner, Observer { event ->
                event.getContentIfNotHandled().notNull {printExpeditionList(it) }
            })

            response.observe(viewLifecycleOwner, Observer {
                if (it.isError){
                    ma.messageWithSound(it.errorMessage,true,false)
                  /*  customDialog.setTitle("Error").setDescription(it.errorMessage).setOkButton("Cerrar"){
                        customDialog.dismiss()
                    }.show()*/
                }else{
                    requireActivity().onBackPressed()
                }
            })
        }
    }

    private fun printExpeditionList(it: ItemPalletViewListVO){
        binding.splashProgress.visibility = View.GONE
        val lm = LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)
        adapter = ExpeditionPalletViewAdapter(it.list)
        binding.expeditionPalletRecyclerview.adapter = adapter
        binding.expeditionPalletRecyclerview.layoutManager = lm
    }

}