package es.verdnatura.presentation.view.feature.paletizador.fragment

import android.content.Context
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.Transformations
import es.verdnatura.domain.GetPaletizadoresUserCase
import es.verdnatura.presentation.base.BaseViewModel

import es.verdnatura.presentation.base.getMessageFromAllResponse
import es.verdnatura.presentation.base.nameofFunction
import es.verdnatura.presentation.common.Event
import es.verdnatura.presentation.common.ResponseItemVO
import es.verdnatura.presentation.view.feature.paletizador.model.ItemPalletViewListVO
import es.verdnatura.presentation.view.feature.paletizador.model.ItemPalletViewVO
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response

class ExpeditionPalletDetailViewModel(context: Context): BaseViewModel() {

    private val getPaletizadoresUserCase: GetPaletizadoresUserCase = GetPaletizadoresUserCase(context)

    private val _expeditionPalletList by lazy { MutableLiveData<ItemPalletViewListVO>() }
    val loadExpeditionPalletList = Transformations.map(_expeditionPalletList) { Event(it) }

    private val _response by lazy { MutableLiveData<ResponseItemVO>() }
    val response: LiveData<ResponseItemVO>
        get() = _response

    fun expeditionPallet_List(usuario:String,password:String,vPalletFk:String,callFunction:String){
        getPaletizadoresUserCase.expeditionPalletView(usuario,password,vPalletFk).enqueue(object :
            Callback<List<ItemPalletViewVO>> {
            override fun onFailure(call: Call<List<ItemPalletViewVO>>, t: Throwable) {
                val listError:ArrayList<ItemPalletViewVO> = ArrayList()
                listError.add(ItemPalletViewVO(isError = true,errorMessage =getMessageFromAllResponse(nameofFunction(this),t.message!!)))
                _expeditionPalletList.value = ItemPalletViewListVO(listError)
            }

            override fun onResponse(
                call: Call<List<ItemPalletViewVO>>,
                response: Response<List<ItemPalletViewVO>>
            ) {
                if (response.body() != null){
                    _expeditionPalletList.value = response.body()?.let { ItemPalletViewListVO(it) }
                }else{
                    val listError:ArrayList<ItemPalletViewVO> = ArrayList()
                    listError.add(ItemPalletViewVO(isError = true,errorMessage = getMessageFromAllResponse(nameofFunction(this),response.message())))
                    _expeditionPalletList.value = ItemPalletViewListVO(listError)
                }
            }

        })
    }

    fun expeditionPalletDel(usuario:String,password:String,vPalletFk:String,callFunction:String){
        getPaletizadoresUserCase.expeditionPalletDel(usuario,password,vPalletFk).enqueue(object : Callback<String>{
            override fun onFailure(call: Call<String>, t: Throwable) {
                _response.value = ResponseItemVO(isError = true,errorMessage = getMessageFromAllResponse(nameofFunction(this),t.message!!))
            }
            override fun onResponse(call: Call<String>, response: Response<String>) {
                if (response.body() == null){
                    _response.value = ResponseItemVO(isError = true,errorMessage = getMessageFromAllResponse(nameofFunction(this),response.message()))
                }else{
                    _response.value = ResponseItemVO(isError = false,response = response.body()!!)
                }
            }
        })
    }

    fun expeditionPalletPrintSet(
        usuario: String,
        password: String,
        vPalletFk: String,
        vSectorFk: String,callFunction:String){

        getPaletizadoresUserCase.expeditionPalletPrintSet(usuario,password,vPalletFk,vSectorFk).enqueue(object : Callback<String>{
            override fun onFailure(call: Call<String>, t: Throwable) {
                _response.value = ResponseItemVO(isError = true,errorMessage =getMessageFromAllResponse(nameofFunction(this),t.message!!))
            }
            override fun onResponse(call: Call<String>, response: Response<String>) {
                if (response.body() == null){
                    _response.value = ResponseItemVO(isError = true,errorMessage = getMessageFromAllResponse(nameofFunction(this),response.message()))
                }else{
                    _response.value = ResponseItemVO(isError = false,response = response.body()!!)
                }
            }
        })
    }


}