package es.verdnatura.presentation.view.feature.pasillero.adapter

import android.content.Context
import android.os.Build
import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.recyclerview.widget.RecyclerView
import es.verdnatura.databinding.ItemPasillerosMainMenuBinding
import es.verdnatura.presentation.common.OnPasillerosItemClickListener
import es.verdnatura.presentation.view.feature.pasillero.model.PasillerosItemVO
import timber.log.Timber.d

class PasillerosAdapter (
    private val items: List<PasillerosItemVO>,
    private val onPasillerosItemClickListener: OnPasillerosItemClickListener
): RecyclerView.Adapter<PasillerosAdapter.PasillerosItemHolder> () {

    var context : Context? = null
    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): PasillerosItemHolder {
        this.context = parent.context
        return PasillerosItemHolder(
            ItemPasillerosMainMenuBinding.inflate(LayoutInflater.from(parent.context),parent,false)
        )
    }

    override fun getItemCount() =items.size

    override fun onBindViewHolder(holder: PasillerosItemHolder, position: Int) {
        holder.bind(items[position])
        holder.binding.root.setOnClickListener {
            onPasillerosItemClickListener.onPasillerosItemClickListener(items[position],"")
        }

    }


    inner class PasillerosItemHolder(
        val binding: ItemPasillerosMainMenuBinding
    ) : RecyclerView.ViewHolder(binding.root){
        fun bind(item: PasillerosItemVO) {
            binding.apply {
                this.item = item
                this.title = context?.getString(item.titleToShow)

                try {
                    itemImage.setImageResource(item.iconResource)
                }catch(exception:Exception){
                    d("Exception::"+exception.message)
                }

                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                    itemImage.tooltipText=item.description
                }
            }
        }
    }
}