package es.verdnatura.presentation.view.feature.presacador.fragment

import android.content.Context
import android.util.Log
import android.util.Log.d
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import es.verdnatura.domain.GetLoginUserCase
import es.verdnatura.domain.GetPreSacadorUseCase
import es.verdnatura.domain.GetSacadorControladorUserCase
import es.verdnatura.domain.GetUbicadorUserCase
import es.verdnatura.presentation.base.*
import es.verdnatura.presentation.common.ResponseItemVO
import es.verdnatura.presentation.view.feature.presacador.model.PreSacadorItemVO
import es.verdnatura.presentation.view.feature.sacador.mapper.toDateFormat
import es.verdnatura.presentation.view.feature.sacador.model.CollectionListVO
import es.verdnatura.presentation.view.feature.sacador.model.CollectionVO
import es.verdnatura.presentation.view.feature.sacador.model.PlacementSupplyListVO
import es.verdnatura.presentation.view.feature.sacador.model.PlacementSupplyVO
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response

class PreSacadorViewModel(context: Context) : BaseViewModel()
{
    private val getPreSacadorUseCase: GetPreSacadorUseCase = GetPreSacadorUseCase(context)
    private val getSacadorControladorUserCase: GetSacadorControladorUserCase =
        GetSacadorControladorUserCase(context)

    private val getUbicadorUserCase: GetUbicadorUserCase = GetUbicadorUserCase(context)
    private val getLoginUserCase: GetLoginUserCase = GetLoginUserCase(context)

    private val _salesList by lazy { MutableLiveData<List<PreSacadorItemVO>>() }
    val salesList: LiveData<List<PreSacadorItemVO>>
        get() = _salesList

    private val _response by lazy { MutableLiveData<ResponseItemVO>() }
    val response: LiveData<ResponseItemVO>
        get() = _response


    private val _responseSaleTrackingUpdate by lazy { MutableLiveData<ResponseItemVO>() }
    val responseSaleTrackingUpdate: LiveData<ResponseItemVO>
        get() = _responseSaleTrackingUpdate

    private val _responseSaleMove by lazy { MutableLiveData<ResponseItemVO>() }
    val responseSaleMove: LiveData<ResponseItemVO>
        get() = _responseSaleMove


    private val _responseParkingAdd by lazy { MutableLiveData<ResponseItemVO>() }
    val responseParkingAdd: LiveData<ResponseItemVO>
        get() = _responseParkingAdd

    private val _placementSuppleyList by lazy { MutableLiveData<PlacementSupplyListVO>() }
    val placementSuppleyList: LiveData<PlacementSupplyListVO>
        get() = _placementSuppleyList

    private val _responseCode by lazy { MutableLiveData<ResponseItemVO>() }
    val responseCode: LiveData<ResponseItemVO>
        get() = _responseCode

    private val _responseIncQuantity by lazy { MutableLiveData<ResponseItemVO>() }
    val responseIncQuantity: LiveData<ResponseItemVO>
        get() = _responseIncQuantity

    private val _responseSalixMessage by lazy { MutableLiveData<ResponseItemVO>() }
    val responseSalixMessage: LiveData<ResponseItemVO>
        get() = _responseSalixMessage

    private val _responseMissingTrash by lazy { MutableLiveData<ResponseItemVO>() }
    val responseMissingTrash: LiveData<ResponseItemVO>
        get() = _responseMissingTrash


    //sergio: previa nuevo
    private val _collectionList by lazy { MutableLiveData<CollectionListVO>() }
    val collectionList: LiveData<CollectionListVO>
        get() = _collectionList

    private val _responseNew by lazy { MutableLiveData<ResponseItemVO>() }
    val responseNew: LiveData<ResponseItemVO>
        get() = _responseNew

    private val _responseSaleGroupAdd by lazy { MutableLiveData<ResponseItemVO>() }
    val responseSaleGroupAdd: LiveData<ResponseItemVO>
        get() = _responseSaleGroupAdd

    private val _responseSaleAddPrevOK by lazy { MutableLiveData<ResponseItemVO>() }
    val responseSaleAddPrevOK: LiveData<ResponseItemVO>
        get() = _responseSaleAddPrevOK

    private val _responseSaleupdateIsChecked by lazy { MutableLiveData<ResponseItemVO>() }
    val responseSaleupdateIsChecked: LiveData<ResponseItemVO>
        get() = _responseSaleupdateIsChecked


    fun ticketToPrePrepare(usuario: String, password: String, ticketFk: String, sectorFk: String)
    {
        getPreSacadorUseCase.ticketToPrePrepare(usuario, password, ticketFk, sectorFk)
            .enqueue(object :
                Callback<List<PreSacadorItemVO>>
            {
                override fun onFailure(call: Call<List<PreSacadorItemVO>>, t: Throwable)
                {
                    d("VERDNATURA::","ha fallado")
                    _salesList.value = listOf()
                }

                override fun onResponse(
                    call: Call<List<PreSacadorItemVO>>,
                    response: Response<List<PreSacadorItemVO>>
                                       )
                {
                    if (response.body() != null)
                    {
                        d("VERDNATURA::","entra con lineas")
                        _salesList.value = response.body()?.let { it }
                    } else
                    {
                        d("VERDNATURA::","ES NULO")
                        _salesList.value = listOf()
                    }
                }
            })
    }

    fun collectionStickerPrint(
        usuario: String,
        password: String,
        collectionFk: String,
        sectorFk: String,callFunction:String
                              )
    {
        getSacadorControladorUserCase.collectionStickerPrint(
            usuario,
            password,
            collectionFk,
            sectorFk
                                                            ).enqueue(object : Callback<String>
        {
            override fun onFailure(call: Call<String>, t: Throwable)
            {
                _response.value = ResponseItemVO(
                    isError = true,
                    errorMessage = getMessageFromAllResponse(nameofFunction(this),t.message!!)
                                                )
            }

            override fun onResponse(call: Call<String>, response: Response<String>)
            {
                if (response.body() == null)
                {
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this),response.message())
                                                    )
                } else
                {
                    _response.value = ResponseItemVO(isError = false, response = response.body()!!)
                }
            }
        })
    }

    fun parking(usuario: String, password: String, ticketFk: String, parking: String,callFunction:String)
    {
        getUbicadorUserCase.shelvingPark(usuario, password, ticketFk, parking)
            .enqueue(object : Callback<String>
            {
                override fun onFailure(call: Call<String>, t: Throwable)
                {
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this),t.message!!)
                                                    )
                }

                override fun onResponse(call: Call<String>, response: Response<String>)
                {
                    if (response.body() == null)
                    {
                        _response.value = ResponseItemVO(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(nameofFunction(this),response.message())
                                                        )
                    } else
                    {
                        _response.value =
                            ResponseItemVO(isError = false, response = response.body()!!)
                    }
                }
            })
    }


    fun saleParking_add(usuario: String, password: String, sales:List<String>,parking: String,callFunction:String)
    {
        getUbicadorUserCase.saleParking_add(usuario, password, sales, parking)
            .enqueue(object : Callback<String>
            {
                override fun onFailure(call: Call<String>, t: Throwable)
                {
                    _responseParkingAdd.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this),t.message!!)
                    )
                }

                override fun onResponse(call: Call<String>, response: Response<String>)
                {
                    if (response.body() == null)
                    {
                        _responseParkingAdd.value = ResponseItemVO(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(nameofFunction(this),response.message())
                        )
                    } else
                    {
                        _responseParkingAdd.value =
                            ResponseItemVO(isError = false, response = response.body()!!)
                    }
                }
            })
    }

    fun itemPlacementSupplyAiming(
        usuario: String,
        password: String,
        shelvingFk: String,
        quantity: String,
        itemFk: String,callFunction:String
                                 )
    {
        getSacadorControladorUserCase.itemPlacementSupplyAiming(
            usuario,
            password,
            shelvingFk,
            quantity,
            itemFk
                                                               ).enqueue(object :
            Callback<List<PlacementSupplyVO>>
        {
            override fun onFailure(call: Call<List<PlacementSupplyVO>>, t: Throwable)
            {
                val listError: ArrayList<PlacementSupplyVO> = ArrayList()
                listError.add(PlacementSupplyVO(isError = true, errorMessage = getMessageFromAllResponse(nameofFunction(this),t.message!!)))
                _placementSuppleyList.value = PlacementSupplyListVO(listError)
            }

            override fun onResponse(
                call: Call<List<PlacementSupplyVO>>,
                response: Response<List<PlacementSupplyVO>>
                                   )
            {
                if (response.body() != null)
                {
                    _placementSuppleyList.value = response.body()?.let { PlacementSupplyListVO(it) }
                } else
                {
                    val listError: ArrayList<PlacementSupplyVO> = ArrayList()
                    listError.add(
                        PlacementSupplyVO(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(nameofFunction(this),response.message())
                                         )
                                 )
                    _placementSuppleyList.value = PlacementSupplyListVO(listError)
                }
            }
        })
    }

    fun itemShelvingSaleSupplyAdd(
        usuario: String,
        password: String,
        itemShelvingFk: String,
        saleFk: String,
        quantity: String,callFunction:String
                                 )
    {
        getSacadorControladorUserCase.itemShelvingSaleSupplyAdd(
            usuario,
            password,
            itemShelvingFk,
            saleFk,
            quantity
                                                               ).enqueue(object : Callback<String>
        {
            override fun onFailure(call: Call<String>, t: Throwable)
            {
                _response.value = ResponseItemVO(isError = true, errorMessage = getMessageFromAllResponse(nameofFunction(this),t.message!!))
            }

            override fun onResponse(call: Call<String>, response: Response<String>)
            {
                if (response.body() == null)
                {
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this),response.message())
                                                    )
                } else
                {
                    _response.value = ResponseItemVO(isError = false, response = response.body()!!)
                }
            }
        })
    }

    fun saleTrackingReplace(
        usuario: String,
        password: String,
        saleFk: String,
        vOriginalQuantity: String,
        vStateFk: String,
        vIsChecked: String,
        vBuyFk: String,callFunction:String
    )
    {
        //Lo ultimo que se envia
        //sergio: se quita el 0 que habia por defecto para la previa  se añada el buyFk que sera el item escaneado.
        //getSacadorControladorUserCase.saleTrackingReplace(usuario,password,saleFk,vOriginalQuantity,vStateFk,vIsChecked,"0").enqueue(object : Callback<String>{
        getSacadorControladorUserCase.saleTrackingReplace(
            usuario,
            password,
            saleFk,
            vOriginalQuantity,
            vStateFk,
            vIsChecked,
            vBuyFk
        ).enqueue(object : Callback<String>
        {
            override fun onFailure(call: Call<String>, t: Throwable)
            {
                _response.value = ResponseItemVO(
                    isError = true,
                    errorMessage = getMessageFromAllResponse(nameofFunction(this),t.message!!)
                )
            }

            override fun onResponse(call: Call<String>, response: Response<String>)
            {
                if (response.body() == null)
                {
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this),response.message())
                    )
                } else
                {
                    _response.value = ResponseItemVO(isError = false, response = response.body()!!)
                }
            }
        })
    }

    fun getIdFromCode(usuario: String, password: String, code: String,callFunction:String)
    {
        getSacadorControladorUserCase.getIdFromCode(usuario, password, code)
            .enqueue(object : Callback<String>
            {
                override fun onFailure(call: Call<String>, t: Throwable)
                {
                    _responseCode.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this),t.message!!)
                                                        )
                }

                override fun onResponse(call: Call<String>, response: Response<String>)
                {
                    if (response.body() == null)
                    {
                        _responseCode.value = ResponseItemVO(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(nameofFunction(this),response.message()))

                    } else
                    {
                        _responseCode.value =
                            ResponseItemVO(isError = false, response = response.body()!!)
                    }
                }
            })
    }

    fun saleTrackingDel(usuario: String, password: String, saleFk: String,callFunction:String){
        getSacadorControladorUserCase.saleTrackingDel(usuario, password, saleFk)
            .enqueue(object : Callback<String>
            {
                override fun onFailure(call: Call<String>, t: Throwable)
                {
                    _response.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this),t.message!!)
                                                    )
                }

                override fun onResponse(call: Call<String>, response: Response<String>)
                {
                    if (response.body() == null)
                    {
                        _response.value = ResponseItemVO(
                            isError = true,
                            errorMessage = getMessageFromAllResponse(nameofFunction(this),response.message()))
                    } else
                    {
                        _response.value =
                            ResponseItemVO(isError = false, response = response.body()!!)
                    }
                }
            })
    }

    fun saleMove(
        usuario: String,
        password: String,
        saleFk: String,
        quantity: String,
        originalQuantity: String,callFunction:String)
    {
        getSacadorControladorUserCase.saleMove(
            usuario,
            password,
            saleFk,
            quantity,
            originalQuantity
                                              ).enqueue(object : Callback<String>
        {
            override fun onFailure(call: Call<String>, t: Throwable)
            {
                _responseSaleMove.value = ResponseItemVO(
                    isError = true,
                    errorMessage = getMessageFromAllResponse(nameofFunction(this),t.message!!)
                                                )
            }

            override fun onResponse(call: Call<String>, response: Response<String>)
            {
                if (response.body() == null)
                {
                    _responseSaleMove.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this),response.message()))
                } else
                {
                    _responseSaleMove.value = ResponseItemVO(isError = false, response = response.body()!!)
                }
            }
        })
    }

    fun collectionMissingTrash(
        usuario: String,
        password: String,
        saleFk: String,
        quantity: String,
        type: String,
        warehouseFk: String,
        originalQuantity: String,callFunction:String)
    {
        getSacadorControladorUserCase.collectionMissingTrash(
            usuario,
            password,
            saleFk,
            quantity,
            type,
            warehouseFk,
            originalQuantity
                                                            ).enqueue(object : Callback<String>
        {
            override fun onFailure(call: Call<String>, t: Throwable)
            {
                _responseMissingTrash.value = ResponseItemVO(
                    isError = true,
                    errorMessage = getMessageFromAllResponse(nameofFunction(this),t.message!!)
                                                )
            }

            override fun onResponse(call: Call<String>, response: Response<String>)
            {
                if (response.body() == null)
                {
                    _responseMissingTrash.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this),response.message()))
                } else
                {
                    _responseMissingTrash.value = ResponseItemVO(isError = false, response = response.body()!!)
                }
            }
        })
    }

    fun collectionIncreaseQuantity(
        usuario: String,
        password: String,
        saleFk: String,
        quantity: String,callFunction:String)
    {
        getSacadorControladorUserCase.collectionIncreaseQuantity(
            usuario,
            password,
            saleFk,
            quantity
                                                                ).enqueue(object : Callback<String>
        {
            override fun onFailure(call: Call<String>, t: Throwable)
            {
                _responseIncQuantity.value = ResponseItemVO(
                    isError = true,
                    errorMessage = getMessageFromAllResponse(nameofFunction(this),t.message!!)
                                                           )
            }

            override fun onResponse(call: Call<String>, response: Response<String>)
            {
                if (response.body() == null)
                {
                    _responseIncQuantity.value = ResponseItemVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this),response.message()))
                } else
                {
                    _responseIncQuantity.value =
                        ResponseItemVO(isError = false, response = response.body()!!)
                }
            }
        })
    }

    fun sendChekingPresence(
        token: String,
        workerId: String,
        message: String,
        calledFunction: String
                           )
    {
        try
        {
            getLoginUserCase.sendChekingPresence(token, workerId.toInt(), message)
                .enqueue(object : Callback<Boolean>
                {
                    override fun onResponse(call: Call<Boolean>, response: Response<Boolean>)
                    {
                        if (!response.isSuccessful)
                        {
                            _responseSalixMessage.value = ResponseItemVO(
                                isError = true,
                                errorMessage = "Error en la llamada $calledFunction al enviar mensaje rocket." + getMessageFromAllResponse(
                                    nameofFunction(this),response.message()
                                                                                                                                       )
                                                                        )
                            //Log.i("Salix","Mensaje enviado a salix")
                        }

                    }

                    override fun onFailure(call: Call<Boolean>, t: Throwable)
                    {
                       /* _responseSalixMessage.value = ResponseItemVO(
                            isError = true,
                            errorMessage = "Error en la llamada $calledFunction al enviar mensaje rocket." + getMessageFromAllResponse(
                                nameofFunction(this),t.message!!))*/

                    }
                })
        } catch (e: Exception)
        {
            Log.d("VERDNATURA","error "+e.message)
        }

    }

    fun sectorCollection_get(usuario: String, password: String,callFunction:String){

        getPreSacadorUseCase.sectorCollection_get(usuario, password).enqueue(object :
            Callback<List<CollectionVO>> {
            override fun onFailure(call: Call<List<CollectionVO>>, t: Throwable) {
                val listError: ArrayList<CollectionVO> = ArrayList()
                listError.add(CollectionVO(isError = true, errorMessage = getMessageFromAllResponse(nameofFunction(this),t.message!!)))
                _collectionList.value = CollectionListVO(listError)
            }


            override fun onResponse(
                call: Call<List<CollectionVO>>,
                response: Response<List<CollectionVO>>
            ) {
                if (response.body() != null) {
                    _collectionList.value =
                        response.body()?.let { CollectionListVO(it.toDateFormat()) }
                } else {
                    val listError: ArrayList<CollectionVO> = ArrayList()
                    listError.add(
                        CollectionVO(
                            isError = true,
                            errorMessage =  getMessageFromAllResponse(nameofFunction(this),response.message())
                        )
                    )
                    _collectionList.value = CollectionListVO(listError)

                }
            }

        })
    }

    fun sectorCollection_new(usuario: String, password: String, sectorFk: String,callFunction:String){
        getPreSacadorUseCase.sectorCollection_new(usuario, password, sectorFk).enqueue(
            object :
                Callback<Unit> {
                override fun onFailure(call: Call<Unit>, t: Throwable) {
                    _responseNew.value =
                        ResponseItemVO(isError = true, errorMessage =  getMessageFromAllResponse(nameofFunction(this),t.message!!))


                }

                override fun onResponse(
                    call: Call<Unit>,
                    response: Response<Unit>
                ) {

                    if (!response.isSuccessful){
                        _responseNew.value=ResponseItemVO(
                            isError = true,
                            errorMessage = "Error en la llamada al añadir colección." + getMessageFromAllResponse(
                                nameofFunction(this),response.message()
                            ))
                    }else{
                        _responseNew.value=ResponseItemVO(
                            isError = false,
                            errorMessage = "" + getMessageFromAllResponse(
                                nameofFunction(this),response.message()
                            ))

                    }


                        //Log.i("Salix","Mensaje enviado a salix")


                    /*if (response.body() != null) {
                        _response.value =
                            ResponseItemVO(isError = false, response = response.body()!!)
                    } else {

                        _response.value = ResponseItemVO(
                            isError = true,
                            errorMessage =  getMessageFromAllResponse(nameofFunction(this),response.message())
                        )

                    }*/
                }
            })
    }

    fun sectorCollection_getSale(usuario: String, password: String, sectorCollectionFK:String,vSectorFk:String)
    {
        getPreSacadorUseCase.sectorCollection_getSale(usuario, password, sectorCollectionFK,vSectorFk)
            .enqueue(object :
                Callback<List<PreSacadorItemVO>>
            {
                override fun onFailure(call: Call<List<PreSacadorItemVO>>, t: Throwable)
                {
                    d("VERDNATURA::","ha fallado")
                    _salesList.value = listOf()
                }

                override fun onResponse(
                    call: Call<List<PreSacadorItemVO>>,
                    response: Response<List<PreSacadorItemVO>>
                )
                {
                    if (response.body() != null)
                    {
                        d("VERDNATURA::","entra con lineas")
                        _salesList.value = response.body()?.let { it }
                    } else
                    {
                        d("VERDNATURA::","ES NULO")
                        _salesList.value = listOf()
                    }
                }
            })
    }

    fun sectorCollectionSaleGroup_add(usuario: String, password: String, vSaleGroupFk: String,vSectorCollectionFk:String,callFunction:String){
        getPreSacadorUseCase.sectorCollectionSaleGroup_add(usuario, password, vSaleGroupFk,vSectorCollectionFk).enqueue(
            object :
                Callback<Unit> {
                override fun onFailure(call: Call<Unit>, t: Throwable) {
                    _response.value =
                        ResponseItemVO(isError = true, errorMessage =  getMessageFromAllResponse(nameofFunction(this),t.message!!))


                }

                override fun onResponse(
                    call: Call<Unit>,
                    response: Response<Unit>
                ) {

                    if (!response.isSuccessful){
                        _responseSaleGroupAdd.value=ResponseItemVO(
                            isError = true,
                            errorMessage = "Error al añadir ticket a la colección." + getMessageFromAllResponse(
                                nameofFunction(this),response.message()
                            ))
                    }else{
                        _responseSaleGroupAdd.value=ResponseItemVO(
                            isError = false,
                            errorMessage = "" + getMessageFromAllResponse(
                                nameofFunction(this),response.message()
                            ))

                    }


                }
            })
    }

    fun saleTracking_addPrevOK(usuario: String, password: String, vSectorCollectionFk:String,callFunction:String){
        getPreSacadorUseCase.saleTracking_addPrevOK(usuario, password,vSectorCollectionFk).enqueue(
            object :
                Callback<Unit> {
                override fun onFailure(call: Call<Unit>, t: Throwable) {
                    _responseSaleAddPrevOK.value =
                        ResponseItemVO(isError = true, errorMessage =  getMessageFromAllResponse(nameofFunction(this),t.message!!))


                }

                override fun onResponse(
                    call: Call<Unit>,
                    response: Response<Unit>
                ) {

                    if (!response.isSuccessful){
                        _responseSaleAddPrevOK.value=ResponseItemVO(
                            isError = true,
                            errorMessage = "Error al cambiar el estado a la colección de previa." + getMessageFromAllResponse(
                                nameofFunction(this),response.message()
                            ))
                    }else{
                        _responseSaleAddPrevOK.value=ResponseItemVO(
                            isError = false,
                            errorMessage = ""
                                    + getMessageFromAllResponse(
                                nameofFunction(this),response.message()
                            ))

                    }


                }
            })
    }

    fun saleTracking_updateIsChecked(usuario: String, password: String, vSaleFk:String,vIsChecked:String,callFunction:String){
        getPreSacadorUseCase.saleTracking_updateIsChecked(usuario, password,vSaleFk,vIsChecked).enqueue(
            object :
                Callback<Unit> {
                override fun onFailure(call: Call<Unit>, t: Throwable) {
                    _responseSaleupdateIsChecked.value =
                        ResponseItemVO(isError = true, errorMessage =  getMessageFromAllResponse(nameofFunction(this),t.message!!))


                }

                override fun onResponse(
                    call: Call<Unit>,
                    response: Response<Unit>
                ) {

                    if (!response.isSuccessful){
                        _responseSaleupdateIsChecked.value=ResponseItemVO(
                            isError = true,
                            errorMessage = "Error al cambiar el estado a la colección de previa." + getMessageFromAllResponse(
                                nameofFunction(this),response.message()
                            ))
                    }else{
                        _responseSaleupdateIsChecked.value=ResponseItemVO(
                            isError = false,
                            errorMessage = ""
                                    + getMessageFromAllResponse(
                                nameofFunction(this),response.message()
                            ))

                    }


                }
            })
    }

}
