package es.verdnatura.presentation.view.feature.sacador.fragment


import android.content.Context
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import es.verdnatura.domain.GetSacadorControladorUserCase
import es.verdnatura.presentation.base.BaseViewModel

import es.verdnatura.presentation.base.getMessageFromAllResponse
import es.verdnatura.presentation.base.nameofFunction
import es.verdnatura.presentation.common.ResponseItemVO
import es.verdnatura.presentation.view.feature.collection.mapper.map
import es.verdnatura.presentation.view.feature.sacador.mapper.toDateFormat
import es.verdnatura.presentation.view.feature.sacador.model.CollectionListVO
import es.verdnatura.presentation.view.feature.sacador.model.CollectionVO
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response

class SacadorViewModel(context: Context) : BaseViewModel() {

    val emptyMessage = "No tienes colecciones pendientes. Presiona sobre el + para crear colección"

    private val getSacadorControladorUserCase: GetSacadorControladorUserCase = GetSacadorControladorUserCase(
        context
    )

    private val _collectionList by lazy { MutableLiveData<CollectionListVO>() }
    val collectionList: LiveData<CollectionListVO>
        get() = _collectionList

    private val _collectionTicketList by lazy { MutableLiveData<CollectionVO>() }
    val collectionTicketList: LiveData<CollectionVO>
        get() = _collectionTicketList

    private val _response by lazy { MutableLiveData<ResponseItemVO>() }
    val response: LiveData<ResponseItemVO>
        get() = _response



    fun collectionTicketGet(
        usuario: String,
        password: String,
        collectionFk: String,
        sectorFk: String,
        print: String,
        type:String,callFunction:String
    ){
        getSacadorControladorUserCase.collectionTicketGet(
            usuario,
            password,
            collectionFk,
            sectorFk,
            print,
            type
        ).enqueue(object :
            Callback<CollectionVO> {
            override fun onFailure(call: Call<CollectionVO>, t: Throwable) {
                _collectionTicketList.value = CollectionVO(
                    isError = true,
                    errorMessage =getMessageFromAllResponse(nameofFunction(this),t.message!!)
                )
            }

            override fun onResponse(
                call: Call<CollectionVO>,
                response: Response<CollectionVO>
            ) {
                if (response.body() != null) {
                    _collectionTicketList.value = response.body()?.let { it.map() }
                } else {
                    _collectionTicketList.value = CollectionVO(
                        isError = true,
                        errorMessage = getMessageFromAllResponse(nameofFunction(this),response.message())
                    )
                }
            }
        })
    }

         fun collectionNew(usuario: String, password: String, sectorFk: String, wagons: String,callFunction:String){
            getSacadorControladorUserCase.collectionNew(usuario, password, sectorFk, wagons).enqueue(
                object :
                    Callback<String> {
                    override fun onFailure(call: Call<String>, t: Throwable) {
                        _response.value =
                            ResponseItemVO(isError = true, errorMessage = getMessageFromAllResponse(nameofFunction(this),t.message!!))


                    }

                    override fun onResponse(
                        call: Call<String>,
                        response: Response<String>
                    ) {
                        if (response.body() != null) {
                            _response.value =
                                ResponseItemVO(isError = false, response = response.body()!!)
                        } else {

                            _response.value = ResponseItemVO(
                                isError = true,
                                errorMessage = getMessageFromAllResponse(nameofFunction(this),response.message())
                            )

                        }
                    }
                })
    }

    fun collectionNew(usuario: String, password: String, sectorFk: String, wagons: String,tagtype:String,callFunction:String){
        getSacadorControladorUserCase.collectionNew(usuario, password, sectorFk, wagons,tagtype).enqueue(
            object :
                Callback<String> {
                override fun onFailure(call: Call<String>, t: Throwable) {
                    _response.value =
                        ResponseItemVO(isError = true, errorMessage =  getMessageFromAllResponse(nameofFunction(this),t.message!!))


                }

                override fun onResponse(
                    call: Call<String>,
                    response: Response<String>
                ) {
                    if (response.body() != null) {
                        _response.value =
                            ResponseItemVO(isError = false, response = response.body()!!)
                    } else {

                        _response.value = ResponseItemVO(
                            isError = true,
                            errorMessage =  getMessageFromAllResponse(nameofFunction(this),response.message())
                        )

                    }
                }
            })
    }


    fun collectionGet(usuario: String, password: String,callFunction:String){
        //////Log.i("VERDNATURA:","collectionGet para coger tickets")
        getSacadorControladorUserCase.collectionGet(usuario, password).enqueue(object :
            Callback<List<CollectionVO>> {
            override fun onFailure(call: Call<List<CollectionVO>>, t: Throwable) {
                val listError: ArrayList<CollectionVO> = ArrayList()
                listError.add(CollectionVO(isError = true, errorMessage = getMessageFromAllResponse(nameofFunction(this),t.message!!)))
                _collectionList.value = CollectionListVO(listError)
            }


            override fun onResponse(
                call: Call<List<CollectionVO>>,
                response: Response<List<CollectionVO>>
            ) {
                if (response.body() != null) {
                    _collectionList.value =
                        response.body()?.let { CollectionListVO(it.toDateFormat()) }
                } else {
                    val listError: ArrayList<CollectionVO> = ArrayList()
                    listError.add(
                        CollectionVO(
                            isError = true,
                            errorMessage =  getMessageFromAllResponse(nameofFunction(this),response.message())
                        )
                    )
                    _collectionList.value = CollectionListVO(listError)

                }
            }

        })
    }

    fun collectionGetSalix(token:String){
        getSacadorControladorUserCase.collectionGetSalix(token).enqueue(object :
            Callback<List<CollectionVO>> {
            override fun onFailure(call: Call<List<CollectionVO>>, t: Throwable) {
                val listError: ArrayList<CollectionVO> = ArrayList()
                listError.add(CollectionVO(isError = true, errorMessage = getMessageFromAllResponse(nameofFunction(this),t.message!!)))
                _collectionList.value = CollectionListVO(listError)
            }


            override fun onResponse(
                call: Call<List<CollectionVO>>,
                response: Response<List<CollectionVO>>
            ) {
                if (response.body() != null) {
                    _collectionList.value =
                        response.body()?.let { CollectionListVO(it.toDateFormat()) }
                } else {
                    val listError: ArrayList<CollectionVO> = ArrayList()
                    listError.add(
                        CollectionVO(
                            isError = true,
                            errorMessage =  getMessageFromAllResponse(nameofFunction(this),response.message())
                        )
                    )
                    _collectionList.value = CollectionListVO(listError)

                }
            }

        })
    }
}
