package es.verdnatura.presentation.view.feature.ubicador.fragment

import android.content.Context
import android.content.SharedPreferences
import android.graphics.drawable.Drawable
import android.media.MediaPlayer
import android.os.Build
import android.os.Bundle
import android.view.View
import android.view.inputmethod.EditorInfo
import android.widget.ImageView
import android.widget.Toast
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import es.verdnatura.R
import es.verdnatura.databinding.FragmentUbicadorBinding
import es.verdnatura.domain.notNull
import es.verdnatura.domain.toast
import es.verdnatura.presentation.base.BaseFragment
import es.verdnatura.presentation.common.*
import es.verdnatura.presentation.view.component.CustomDialog
import es.verdnatura.presentation.view.component.CustomDialogInput
import es.verdnatura.presentation.view.component.CustomDialogTwoButtons
import es.verdnatura.presentation.view.component.CustomDialogUbicador
import es.verdnatura.presentation.view.feature.main.activity.MainActivity
import es.verdnatura.presentation.view.feature.pasillero.model.PasillerosItemVO
import es.verdnatura.presentation.view.feature.ubicador.adapter.UbicadorAdapter
import es.verdnatura.presentation.view.feature.ubicador.model.ItemUbicadorVO


class UbicadorFragment(
    var shelvingFk: String = ""
) : BaseFragment<FragmentUbicadorBinding, UbicadorViewModel>(
    UbicadorViewModel::class
) {

    private var adapter: UbicadorAdapter? = null
    private lateinit var customDialogInput: CustomDialogInput
    private lateinit var customDialogUbicador: CustomDialogUbicador
    private lateinit var customDialogTwoButtons: CustomDialogTwoButtons
    private var listItems: ArrayList<ItemUbicadorVO> = ArrayList()
    private lateinit var customDialog: CustomDialog
    private var deep: Int = 1
    private var parking: String = ""
    private var etiquetas: String = "0"
    private var pasillerosItemClickListener: OnPasillerosItemClickListener? = null
    var mperror: MediaPlayer? = null
    var mpok: MediaPlayer? = null


    companion object {
        fun newInstance(entryPoint: String) = UbicadorFragment(entryPoint)
    }

    override fun getLayoutId(): Int = R.layout.fragment_ubicador

    override fun onAttach(context: Context) {
        if (context is OnPasillerosItemClickListener) pasillerosItemClickListener = context
        super.onAttach(context)
    }

    override fun init() {

        customDialogInput = CustomDialogInput(requireContext())
        customDialog = CustomDialog(requireContext())
        customDialogUbicador = CustomDialogUbicador(requireContext())
        customDialogTwoButtons = CustomDialogTwoButtons(requireContext())

        ma.hideBottomNavigation(View.GONE)
        binding.mainToolbar.toolbarTitle.text = getString(R.string.locator)
        setEvents()
        setToolBar()
        if (!shelvingFk.isNullOrEmpty()) {
            binding.splashProgress.visibility = View.VISIBLE
            binding.mainToolbar.toolbarTitle.text = shelvingFk
            viewModel.itemShelvingList( getData(USER), getData(PASSWORD), shelvingFk, "itemShelvingList")
        }
        super.init()
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        mperror = MediaPlayer.create((activity as MainActivity), R.raw.error)
        mpok = MediaPlayer.create((activity as MainActivity), R.raw.ok)
        super.onCreate(savedInstanceState)
    }

    private fun setToolBar() {

        /* val listIcons:ArrayList<Drawable> = ArrayList()
         val iconAdd : Drawable = resources.getDrawable(R.drawable.ic_add_black_24dp,resources.newTheme())
         val iconReload : Drawable = resources.getDrawable(R.drawable.ic_autorenew_black_24dp,resources.newTheme())
         listIcons.add(iconAdd)
         listIcons.add(iconReload)*/
        val listIcons: ArrayList<ImageView> = ArrayList()
        val iconAdd = ImageView(context)
        iconAdd.setImageResource(R.drawable.ic_add_black_24dp)
        val iconReload = ImageView(context)
        iconReload.setImageResource(R.drawable.ic_autorenew_black_24dp)

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            iconReload.tooltipText = getTooltip(R.drawable.ic_autorenew_black_24dp)
            iconAdd.tooltipText =  "Añade item al carro"
        }
        listIcons.add(iconAdd)
        listIcons.add(iconReload)
        binding.mainToolbar.toolbarIcons.adapter =
            ToolBarAdapterTooltip(listIcons, object : OnOptionsSelectedListener {
                override fun onOptionsItemSelected(item: Drawable) {

                    when (item) {
                        iconAdd.drawable -> showAddItem("", "", "", "", false, ItemUbicadorVO(), "")
                        iconReload.drawable -> {
                            binding.splashProgress.visibility = View.VISIBLE
                            viewModel.itemShelvingList(
                                getData(USER),
                                getData(PASSWORD),
                                shelvingFk,
                                "itemShelvingList"
                            )
                        }
                    }
                    /*if (item == iconAdd){
                        showAddItem("","","","",false,ItemUbicadorVO(),"")
                    }else if(item == iconReload){
                        binding.splashProgress.visibility = View.VISIBLE
                        viewModel.itemShelvingList(user,password,shelvingFk,"itemShelvingList")
                    }*/
                }

            })
        binding.mainToolbar.toolbarIcons.layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.HORIZONTAL, false)
    }

    private fun setEvents() {
        binding.mainToolbar.backButton.setOnClickListener {
            requireActivity().onBackPressed()
        }

        setTooltips()
        binding.editMatricula.requestFocus()
        binding.editMatricula.setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5) {
                if (!binding.editMatricula.text.toString().isNullOrEmpty()) {
                    binding.mainToolbar.toolbarTitle.text = binding.editMatricula.text.toString()
                    shelvingFk = binding.editMatricula.text.toString()
                    binding.splashProgress.visibility = View.VISIBLE
                    viewModel.itemShelvingList(
                        getData(USER),
                        getData(PASSWORD),
                        binding.editMatricula.text.toString(),
                        "itemShelvingList"
                    )
                }

                binding.editMatricula.setText("")
                ma.hideKeyboard(binding.editMatricula)
                return@setOnEditorActionListener true
            }

            return@setOnEditorActionListener false
        }

        binding.editPrioridad.setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0 || actionId == 5) {
                if (!binding.editPrioridad.text.toString().isNullOrEmpty()) {
                    viewModel.shelvingPriorityUpdate(
                        getData(USER),
                        getData(PASSWORD),
                        binding.editPrioridad.text.toString(),
                        shelvingFk,
                        "shelvingPriority_update"
                    )
                }

                ma.hideKeyboard(binding.editPrioridad)
                return@setOnEditorActionListener true
            }

            return@setOnEditorActionListener false
        }


        binding.automaticImg.setOnClickListener {
            if (!shelvingFk.isNullOrEmpty())
                ma.onPasillerosItemClickListener(
                    PasillerosItemVO(title = "Automatic"),
                    entryPoint = shelvingFk
                )
            else
                "Escanea una matrícula".toast(requireActivity(), Toast.LENGTH_SHORT)
        }

        binding.parkingImg.setOnClickListener {
            customDialogInput.setTitle("Aparcar carro").setValue("").setOkButton("Aparcar") {
                viewModel.shelvingPark(
                    getData(USER),
                    getData(PASSWORD),
                    shelvingFk,
                    customDialogInput.getValue(),
                    "shelvingPark"
                )
                parking = customDialogInput.getValue()
                binding.mainToolbar.toolbarTitle.text =
                    shelvingFk.uppercase() + " P: " + parking + " E: " + etiquetas
                if (mpok != null) mpok!!.start()
                customDialogInput.dismiss()
            }.setKoButton("Cancelar") {
                customDialogInput.dismiss()
            }.show()

            customDialogInput.getEditText().requestFocus()
            ma.hideKeyboard(customDialogInput.getEditText())

            customDialogInput.getEditText().setOnEditorActionListener { v, actionId, event ->
                if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                    viewModel.shelvingPark(
                        getData(USER),
                        getData(PASSWORD),
                        shelvingFk,
                        customDialogInput.getValue(),
                        "shelvingPark"
                    )
                    customDialogInput.dismiss()
                    parking = customDialogInput.getValue()
                    if (binding.mainToolbar.toolbarTitle != null && shelvingFk != null && parking != null && etiquetas != null) {
                        binding.mainToolbar.toolbarTitle.text =
                            shelvingFk.uppercase() + " P: " + parking + " E: " + etiquetas
                    } else {
                        binding.mainToolbar.toolbarTitle.text = ""
                    }
                    if (mpok != null) {
                        mpok!!.start()
                    }

                    return@setOnEditorActionListener true
                }
                false
            }

        }

        binding.editImg.setOnClickListener {
            customDialogInput.setTitle("Cambiar matrícula").setValue("").setOkButton("Guardar") {
                viewModel.shelvingChange(
                    getData(USER),
                    getData(PASSWORD),
                    shelvingFk,
                    customDialogInput.getValue(),
                    "shelvingChange"
                )
                shelvingFk = customDialogInput.getValue()
                binding.mainToolbar.toolbarTitle.text =
                    shelvingFk.uppercase() + " P: " + parking + " E: " + etiquetas
                if (mpok != null) mpok!!.start()
                customDialogInput.dismiss()
            }.setKoButton("Cancelar") {
                customDialogInput.dismiss()
            }.show()

            customDialogInput.getEditText().requestFocus()
            ma.hideKeyboard(customDialogInput.getEditText())

            customDialogInput.getEditText().setOnEditorActionListener { v, actionId, event ->
                if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                    viewModel.shelvingChange(
                        getData(USER),
                        getData(PASSWORD),
                        shelvingFk,
                        customDialogInput.getValue(),
                        "shelvingChange"
                    )
                    shelvingFk = customDialogInput.getValue()
                    binding.mainToolbar.toolbarTitle.text =
                        shelvingFk.uppercase() + " P: " + parking + " E: " + etiquetas
                    if (mpok != null) mpok!!.start()
                    customDialogInput.dismiss()
                    return@setOnEditorActionListener true
                }
                false
            }

        }

        binding.deleteImg.setOnClickListener {
            customDialog.setTitle("Vaciar carro")
                .setDescription("Esta operación no se puede rectificar.").setOkButton("Vaciar") {
                viewModel.clearShelvingList(getData(USER),
                    getData(PASSWORD), shelvingFk, "clearShelvingList")
                listItems.forEach {
                    it.stickers = "0"
                    it.visible = "0"
                }
                if (adapter != null) {
                    adapter!!.notifyDataSetChanged()
                }
                customDialog.dismiss()
            }.setKoButton("Cancelar") {
                customDialog.dismiss()
            }.show()


        }

    }

    private fun setTooltips() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            binding.editImg.tooltipText="Permite cambiar la matrícula al carro"
            binding.deleteImg.tooltipText="Borra todos los items del carro"
            binding.editPrioridad.tooltipText="Cambia la prioridad de los items del carro. Cuanto más alta antes cogerán los sacadores."
            binding.parkingImg.tooltipText="Permite aparcar el carro"
            binding.automaticImg.tooltipText="Permite añadir items al carro de manera continuada. No uno a uno"

        }
        }

    override fun observeViewModel() {
        with(viewModel) {
            loadShelvingList.observe(viewLifecycleOwner, Observer { event ->
                event.getContentIfNotHandled().notNull {
                    deep = it.list.size
                    listItems = ArrayList()
                    var contEtiquetas: Int = 0
                    it.list.forEach { item ->
                        try {
                            contEtiquetas += item.stickers.toInt()
                        } catch (e: Exception) {
                        }

                        parking = item.code.uppercase()
                        listItems.add(item)
                    }
                    etiquetas = contEtiquetas.toString()
                    binding.mainToolbar.toolbarTitle.text =
                        shelvingFk.uppercase() + " P: " + parking + " E: " + etiquetas
                    if (it.list.size > 0)
                        binding.editPrioridad.setText(it.list[0].priority)
                    binding.splashProgress.visibility = View.GONE

                    //sergio: para mostrar CustomDialogUbicator, ubicador con grouping
                    adapter = UbicadorAdapter(listItems,
                        onPasillerosItemClickListener = pasillerosItemClickListener!!,
                        onVisibleClickListener = object : OnVisibleClickListener {
                            override fun onVisibleClickListener(item: ItemUbicadorVO) {

                                // if (item.grouping.isNullOrEmpty()){item.grouping=""}
                                showAddItem(
                                    item.item,
                                    item.packing,
                                    item.stickers,
                                    item.visible,
                                    true,
                                    item,
                                    grouping = if (item.grouping.isNullOrEmpty()) {
                                        ""
                                    } else item.grouping
                                )

                            }

                        },
                        onMoreClickListener = object : OnMoreClickListener {
                            override fun onMoreClickListener(item: ItemUbicadorVO) {
                                showMoreOptions(item)
                            }

                        })

                    binding.locationRecyclerview.adapter = adapter
                    binding.locationRecyclerview.layoutManager =
                        LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, true)

                }
            })

            response.observe(viewLifecycleOwner, Observer {

             if (it.isError) {
                ma.messageWithSound(it.errorMessage,it.isError,false)

                /*    mperror!!.start()
                    customDialog.setTitle("Error").setDescription(it.errorMessage)
                        .setOkButton("Cerrar") {
                            customDialog.dismiss()
                        }.show()*/
                }
            })

            responsepriority.observe(viewLifecycleOwner, Observer {

                ma.messageWithSound(
                    if (it.isError) it.errorMessage else{it.response},it.isError,true)

               /* if (it.isError) {
                    mperror!!.start()
                    customDialog.setTitle("Error").setDescription(it.errorMessage)
                        .setOkButton("Cerrar") {
                            customDialog.dismiss()
                        }.show()
                } else {
                    (it.response).toast(requireContext())
                }*/
            })

            responseMake.observe(viewLifecycleOwner, Observer {
                if (!shelvingFk.isNullOrEmpty()) {
                    binding.splashProgress.visibility = View.VISIBLE
                    binding.mainToolbar.toolbarTitle.text = shelvingFk
                    viewModel.itemShelvingList(getData(USER),
                        getData(PASSWORD), shelvingFk, "itemShelvingList")
                }
            })
        }
    }

    private fun showAddItem(
        itemFk: String,
        packing: String,
        etiquetas: String,
        visible: String,
        delete: Boolean,
        itemUbicador: ItemUbicadorVO,
        grouping: String
    ) {
        var title = "Nuevo Item"
        if (delete) title = "Editar item " + itemFk
        customDialogUbicador.setTitle(title).setItemValue(itemFk).setPackingValue(packing)
            .setGroupingValue(grouping).setEtiquetaValue(etiquetas).setVisibleValue(visible)
            .setOkButton("Guardar") {
                var packingUpdate = false
                if (packing != customDialogUbicador.getPackingValue() || etiquetas != customDialogUbicador.getEtiquetaValue()) {
                    packingUpdate = true
                }
                saveNewItem(
                    customDialogUbicador.getItemValue(),
                    customDialogUbicador.getEtiquetaValue(),
                    customDialogUbicador.getPackingValue(),
                    customDialogUbicador.getVisibleValue(),
                    if (delete) packingUpdate else null,
                    itemUbicador.id,
                    customDialogUbicador.getGroupingValue()
                )
            }.setKoButton("Cerrar") {
            customDialogUbicador.dismiss()
        }.show()

        customDialogUbicador.getEditItem().requestFocus()
        ma.hideKeyboard(customDialogUbicador.getEditItem())

        customDialogUbicador.getVisibleItem().setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                var packingUpdate = false
                if (packing != customDialogUbicador.getPackingValue() || etiquetas != customDialogUbicador.getEtiquetaValue()) {
                    packingUpdate = true
                }
                saveNewItem(
                    customDialogUbicador.getItemValue(),
                    customDialogUbicador.getEtiquetaValue(),
                    customDialogUbicador.getPackingValue(),
                    customDialogUbicador.getVisibleValue(),
                    if (delete) packingUpdate else null,
                    itemUbicador.id,
                    customDialogUbicador.getGroupingValue()
                )
                return@setOnEditorActionListener true
            }
            false
        }

        customDialogUbicador.getEtiquetaItem().setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                var packingUpdate = false
                if (packing != customDialogUbicador.getPackingValue() || etiquetas != customDialogUbicador.getEtiquetaValue()) {
                    packingUpdate = true
                }
                saveNewItem(
                    customDialogUbicador.getItemValue(),
                    customDialogUbicador.getEtiquetaValue(),
                    customDialogUbicador.getPackingValue(),
                    customDialogUbicador.getVisibleValue(),
                    if (delete) packingUpdate else null,
                    itemUbicador.id,
                    customDialogUbicador.getGroupingValue()
                )
                return@setOnEditorActionListener true
            }
            false
        }

    }

    private fun saveNewItem(
        itemFk: String,
        etiquetas: String,
        packing: String,
        visible: String,
        updatePacking: Boolean? = null,
        itemShelvingFk: String,
        groupingPack: String
    ) {
        if (!packing.isNullOrEmpty()) {
            if (updatePacking == null) {
                if (!itemFk.isNullOrEmpty() && !visible.isNullOrEmpty()) {
                    viewModel.itemShelvingMake(
                        getData(USER),
                        getData(PASSWORD),
                        shelvingFk,
                        itemFk,
                        deep.toString(),
                        visible,
                        packing,
                        getData(WAREHOUSEFK),
                        "1",
                        "itemShelvingMake",
                        groupingPack
                    )
                    changeOfflineValue(
                        ItemUbicadorVO(
                            item = itemFk,
                            visible = visible,
                            packing = packing
                        )
                    )
                    customDialogUbicador.setItemValue("").setPackingValue("").setGroupingValue("")
                        .setEtiquetaValue("").setVisibleValue("")
                    customDialogUbicador.getEditItem().requestFocus()
                    ma.hideKeyboard(customDialogUbicador.getEditItem())
                    customDialogUbicador.dismiss()

                } else if (!itemFk.isNullOrEmpty() && !etiquetas.isNullOrEmpty()) {
                    val visible_calc = (etiquetas.toInt() * packing.toInt()).toString()
                    viewModel.itemShelvingMake(
                        getData(USER),
                        getData(PASSWORD),
                        shelvingFk,
                        itemFk,
                        deep.toString(),
                        visible_calc,
                        packing,
                        getData(WAREHOUSEFK),
                        "1",
                        "itemShelvingMake",
                        groupingPack
                    )
                    changeOfflineValue(
                        ItemUbicadorVO(
                            item = itemFk,
                            visible = visible_calc,
                            packing = packing,
                            stickers = etiquetas
                        )
                    )
                    customDialogUbicador.setItemValue("").setPackingValue("").setGroupingValue("")
                        .setEtiquetaValue("").setVisibleValue("")
                    customDialogUbicador.getEditItem().requestFocus()
                    ma.hideKeyboard(customDialogUbicador.getEditItem())
                    customDialogUbicador.dismiss()

                } else {
                    "Rellena campos obligatorios".toast(
                        (activity as MainActivity),
                        Toast.LENGTH_LONG
                    )
                }
            } else {
                if (!itemFk.isNullOrEmpty() && !visible.isNullOrEmpty() && !updatePacking) {
                    viewModel.itemShelvingMakeEdit(
                        getData(USER),
                        getData(PASSWORD),
                        shelvingFk,
                        itemFk,
                        deep.toString(),
                        visible,
                        packing,
                        getData(WAREHOUSEFK),
                        "1",
                        itemShelvingFk,
                        "itemShelvingMakeEdit",
                        groupingPack
                    )
                    changeOfflineValue(
                        ItemUbicadorVO(
                            item = itemFk,
                            visible = visible,
                            packing = packing
                        )
                    )
                    customDialogUbicador.setItemValue("").setPackingValue("").setGroupingValue("")
                        .setEtiquetaValue("").setVisibleValue("")
                    customDialogUbicador.getEditItem().requestFocus()
                    ma.hideKeyboard(customDialogUbicador.getEditItem())
                    customDialogUbicador.dismiss()

                } else if (!itemFk.isNullOrEmpty() && !etiquetas.isNullOrEmpty() && updatePacking) {
                    var visible_calc = "0"
                    try {
                        visible_calc = (etiquetas.toInt() * packing.toInt()).toString()
                    } catch (e: Exception) {
                    }
                    viewModel.itemShelvingMakeEdit(
                        getData(USER),
                        getData(PASSWORD),
                        shelvingFk,
                        itemFk,
                        deep.toString(),
                        visible_calc,
                        packing,
                        getData(WAREHOUSEFK),
                        "1",
                        itemShelvingFk,
                        "itemShelvingMakeEdit",
                        groupingPack
                    )
                    changeOfflineValue(
                        ItemUbicadorVO(
                            item = itemFk,
                            visible = visible_calc,
                            packing = packing,
                            stickers = etiquetas
                        )
                    )
                    customDialogUbicador.setItemValue("").setPackingValue("").setGroupingValue("")
                        .setEtiquetaValue("").setVisibleValue("")
                    customDialogUbicador.getEditItem().requestFocus()
                    ma.hideKeyboard(customDialogUbicador.getEditItem())
                    customDialogUbicador.dismiss()

                } else {
                    "Rellena campos obligatorios".toast(
                        (activity as MainActivity),
                        Toast.LENGTH_LONG
                    )
                }
            }

        } else {
            "Packing obligatorio".toast((activity as MainActivity), Toast.LENGTH_LONG)
        }
    }

    /* private fun editItem(itemFk:String,etiquetas:String,packing:String,visible:String){
         if (!packing.isNullOrEmpty()){
             if (!itemFk.isNullOrEmpty() && !visible.isNullOrEmpty()){
                 viewModel.itemShelvingMake(user,password,shelvingFk,itemFk,deep.toString(),visible,packing,warehouseFk,"1","itemShelvingMake")
                 changeOfflineValue(ItemUbicadorVO(item = itemFk,visible = visible,packing = packing))
                 customDialogUbicador.setItemValue("").setPackingValue("").setEtiquetaValue("").setVisibleValue("")
                 customDialogUbicador.getEditItem().requestFocus()
                 ma.hideKeyboard(customDialogUbicador.getEditItem())
             }else if(!itemFk.isNullOrEmpty() && !etiquetas.isNullOrEmpty()){
                 val visible_calc = (etiquetas.toInt() * packing.toInt()).toString()
                 viewModel.itemShelvingMake(user,password,shelvingFk,itemFk,deep.toString(),visible_calc,packing,warehouseFk,"1","itemShelvingMake")
                 changeOfflineValue(ItemUbicadorVO(item = itemFk,visible = visible_calc,packing = packing,stickers = etiquetas))
                 customDialogUbicador.setItemValue("").setPackingValue("").setEtiquetaValue("").setVisibleValue("")
                 customDialogUbicador.getEditItem().requestFocus()
                 ma.hideKeyboard(customDialogUbicador.getEditItem())
             }else{
                 "Rellena campos obligatorios".toast((activity as MainActivity), Toast.LENGTH_LONG)
             }
         }else{
             "Packing obligatorio".toast((activity as MainActivity), Toast.LENGTH_LONG)
         }
     }*/

    private fun changeOfflineValue(item: ItemUbicadorVO) {
        if (binding.splashProgress != null) binding.splashProgress.visibility = View.VISIBLE
        /*listItems.add(item)
        adapter?.notifyDataSetChanged()*/
    }

    private fun showMoreOptions(item: ItemUbicadorVO) {
        customDialogTwoButtons.setTitle(item.item).setDescription("Selecciona una acción")
            .setOkButton("Transferir") {
                customDialogInput.setTitle("Nueva matrícula")
                    .setDescription("Introduce el destino para item: " + item.item).setValue("")
                    .setOkButton("Transferir") {
                        if (customDialogInput.getValue().isNotEmpty()) {
                            viewModel.itemShelvingTransfer(
                                getData(USER),
                                getData(PASSWORD),
                                item.id,
                                customDialogInput.getValue(),
                                "itemShelvingTransfer"
                            )
                            listItems.remove(item)
                            adapter!!.notifyDataSetChanged()
                            customDialogInput.dismiss()
                        } else {
                            "Carro incorrecto".toast(requireContext())
                        }


                    }.setKoButton("Cancelar") {
                    customDialogInput.dismiss()
                }.show()
                customDialogInput.getEditText().requestFocus()
                ma.hideKeyboard(customDialogInput.getEditText())
                customDialogInput.getEditText().setOnEditorActionListener { v, actionId, event ->
                    if (actionId == EditorInfo.IME_ACTION_SEARCH || actionId == EditorInfo.IME_ACTION_DONE || actionId == 0) {
                        viewModel.itemShelvingTransfer(
                            getData(USER),
                            getData(PASSWORD),
                            item.id,
                            customDialogInput.getValue(),
                            "itemShelvingTransfer"
                        )
                        listItems.remove(item)
                        adapter!!.notifyDataSetChanged()
                        customDialogInput.dismiss()
                        return@setOnEditorActionListener true
                    }
                    false
                }
                customDialogTwoButtons.dismiss()
            }.setOkButtonTwo("Eliminar") {
            customDialog.setTitle("Confirmar")
                .setDescription("Confirmas eliminar el artículo: " + item.item + "?")
                .setOkButton("Eliminar") {
                    listItems.remove(item)
                    adapter!!.notifyDataSetChanged()
                    viewModel.itemShelvingDelete(getData(USER),
                        getData(PASSWORD), item.id, "itemShelvingDelete")
                    customDialogTwoButtons.dismiss()
                    customDialog.dismiss()
                }.setKoButton("Cancelar") {
                customDialog.dismiss()
                customDialogTwoButtons.dismiss()
            }.show()

        }.setKoButton("Cancelar") {
            customDialogTwoButtons.dismiss()
        }.show()
    }
}